# -*- encoding: utf-8 -*-

from twisted.internet.defer import inlineCallbacks, DeferredQueue

from vumi.utils import http_request
from vumi.message import TransportUserMessage
from vumi.transports.integrat.integrat import (
    IntegratHttpResource, IntegratTransport)
from vumi.transports.tests.helpers import TransportHelper
from vumi.tests.fake_connection import FakeHttpServer
from vumi.tests.helpers import VumiTestCase


XML_TEMPLATE = '''
<Message>
    <Version Version="1.0"/>
    <Response Type="OnUSSEvent">
        <SystemID>Higate</SystemID>
        <UserID>LoginName</UserID>
        <Service>SERVICECODE</Service>
        <Network ID="1" MCC="655" MNC="001"/>
        <OnUSSEvent Type="%(ussd_type)s">
            <USSContext SessionID="%(sid)s"
                        NetworkSID="%(network_sid)s"
                        MSISDN="%(msisdn)s"
                        Script="testscript"
                        ConnStr="%(connstr)s"/>
            <USSText Type="TEXT">%(text)s</USSText>
        </OnUSSEvent>
    </Response>
</Message>
'''


class TestIntegratHttpResource(VumiTestCase):

    DEFAULT_MSG = {
        'from_addr': '+2799053421',
        'to_addr': '*120*44#',
        'transport_metadata': {
            'session_id': 'sess1234',
            },
        }

    def setUp(self):
        self.msgs = []
        resource = IntegratHttpResource("testgrat", "ussd", self._publish)
        self.fake_http = FakeHttpServer(lambda req: resource.render(req))

    def _publish(self, **kws):
        self.msgs.append(kws)

    def send_request(self, xml):
        return http_request(
            "/", xml, method='GET', agent_class=self.fake_http.get_agent)

    @inlineCallbacks
    def check_response(self, xml, responses):
        """Check that sending the given XML results in the given responses."""
        yield self.send_request(xml)
        for msg, expected_override in zip(self.msgs, responses):
            expected = self.DEFAULT_MSG.copy()
            expected.update(expected_override)
            for key, value in expected.items():
                self.assertEqual(msg[key], value)
        self.assertEqual(len(self.msgs), len(responses))
        del self.msgs[:]

    def make_ussd(self, ussd_type, sid=None, network_sid="netsid12345",
                  msisdn=None, connstr=None, text=""):
        sid = (self.DEFAULT_MSG['transport_metadata']['session_id']
               if sid is None else sid)
        msisdn = self.DEFAULT_MSG['from_addr'] if msisdn is None else msisdn
        connstr = self.DEFAULT_MSG['to_addr'] if connstr is None else connstr
        return XML_TEMPLATE % {
            'ussd_type': ussd_type, 'sid': sid, 'network_sid': network_sid,
            'msisdn': msisdn, 'connstr': connstr, 'text': text,
            }

    @inlineCallbacks
    def test_new_session(self):
        # this should not generate a message since we use
        # the 'open' event to start a new sesison.
        xml = self.make_ussd(ussd_type='New', text="")
        yield self.check_response(xml, [])

    @inlineCallbacks
    def test_new_session_via_request(self):
        # this should not generate a message since we use
        # the 'open' event to start a new sesison.
        xml = self.make_ussd(ussd_type='Request', text="REQ")
        yield self.check_response(xml, [])

    @inlineCallbacks
    def test_open_session(self):
        xml = self.make_ussd(ussd_type='Open', text="")
        yield self.check_response(xml, [{
            'session_event': TransportUserMessage.SESSION_NEW,
            'content': None,
            }])

    @inlineCallbacks
    def test_resume_session(self):
        xml = self.make_ussd(ussd_type='Request', text="foo")
        yield self.check_response(xml, [{
            'session_event': TransportUserMessage.SESSION_RESUME,
            'content': 'foo',
            }])

    @inlineCallbacks
    def test_non_ussd(self):
        xml = """
        <Message>
            <Version Version="1.0"/>
            <Response Type="OnReceiveSMS">
            <OnReceiveSMS>
              <Content Type="HEX">06052677F6A565 ...etc</Content>
            </OnReceiveSMS>
            </Response>
        </Message>
        """
        yield self.check_response(xml, [])


class TestIntegratTransport(VumiTestCase):

    @inlineCallbacks
    def setUp(self):
        self.integrat_calls = DeferredQueue()
        self.fake_http = FakeHttpServer(self.handle_request)
        self.base_url = "http://integrat.example.com/"
        config = {
            'web_path': "foo",
            'web_port': "0",
            'url': self.base_url,
            'username': 'testuser',
            'password': 'testpass',
            }
        self.tx_helper = self.add_helper(TransportHelper(IntegratTransport))
        self.transport = yield self.tx_helper.get_transport(config)
        self.transport.agent_factory = self.fake_http.get_agent
        addr = self.transport.web_resource.getHost()
        self.transport_url = "http://%s:%s/" % (addr.host, addr.port)
        self.higate_response = '<Response status_code="0"/>'

    def handle_request(self, request):
        # The content attr will have been set to None by the time we read this.
        request.content_body = request.content.getvalue()
        self.integrat_calls.put(request)
        return self.higate_response

    @inlineCallbacks
    def test_health(self):
        result = yield http_request(self.transport_url + "health", "",
                                    method='GET')
        self.assertEqual(result, "OK")

    @inlineCallbacks
    def test_outbound(self):
        yield self.tx_helper.make_dispatch_outbound("hi", transport_metadata={
            'session_id': "sess123",
        })
        req = yield self.integrat_calls.get()
        self.assertEqual(req.path, self.base_url)
        self.assertEqual(req.method, 'POST')
        self.assertEqual(req.getHeader('content-type'),
                         'text/xml; charset=utf-8')
        self.assertEqual(req.content_body,
                         '<Message><Version Version="1.0" />'
                         '<Request Flags="0" SessionID="sess123"'
                         ' Type="USSReply">'
                         '<UserID Orientation="TR">testuser</UserID>'
                         '<Password>testpass</Password>'
                         '<USSText Type="TEXT">hi</USSText>'
                         '</Request></Message>')

    @inlineCallbacks
    def test_outbound_no_content(self):
        yield self.tx_helper.make_dispatch_outbound(None, transport_metadata={
            'session_id': "sess123",
        })
        req = yield self.integrat_calls.get()
        self.assertEqual(req.path, self.base_url)
        self.assertEqual(req.method, 'POST')
        self.assertEqual(req.getHeader('content-type'),
                         'text/xml; charset=utf-8')
        self.assertEqual(req.content_body,
                         '<Message><Version Version="1.0" />'
                         '<Request Flags="0" SessionID="sess123"'
                         ' Type="USSReply">'
                         '<UserID Orientation="TR">testuser</UserID>'
                         '<Password>testpass</Password>'
                         '<USSText Type="TEXT" />'
                         '</Request></Message>')

    @inlineCallbacks
    def test_inbound(self):
        xml = XML_TEMPLATE % {
            'ussd_type': 'Request',
            'sid': 'sess1234',
            'network_sid': "netsid12345",
            'msisdn': '27345',
            'connstr': '*120*99#',
            'text': 'foobar',
            }
        yield http_request(self.transport_url + "foo", xml, method='GET')
        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg['transport_name'], self.tx_helper.transport_name)
        self.assertEqual(msg['transport_type'], "ussd")
        self.assertEqual(msg['transport_metadata'],
                         {"session_id": "sess1234"})
        self.assertEqual(msg['session_event'],
                         TransportUserMessage.SESSION_RESUME)
        self.assertEqual(msg['from_addr'], '27345')
        self.assertEqual(msg['to_addr'], '*120*99#')
        self.assertEqual(msg['content'], 'foobar')

    @inlineCallbacks
    def test_inbound_non_ascii(self):
        xml = (XML_TEMPLATE % {
            'ussd_type': 'Request',
            'sid': 'sess1234',
            'network_sid': "netsid12345",
            'msisdn': '27345',
            'connstr': '*120*99#',
            'text': u'öæł',
            }).encode("utf-8")
        yield http_request(self.transport_url + "foo", xml, method='GET')
        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg['content'], u'öæł')

    @inlineCallbacks
    def test_nack(self):
        self.higate_response = """
            <Response status_code="-1">
                <Data name="method_error">
                    <field name="error_code" value="-1"/>
                    <field name="reason" value="Expecting POST, not GET"/>
                </Data>
            </Response>""".strip()

        msg = yield self.tx_helper.make_dispatch_outbound(
            "hi", transport_metadata={'session_id': "sess123"})
        yield self.integrat_calls.get()
        [nack] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(nack['user_message_id'], msg['message_id'])
        self.assertEqual(nack['sent_message_id'], msg['message_id'])
        self.assertEqual(nack['nack_reason'],
                         'error_code: -1, reason: Expecting POST, not GET')



# -*- test-case-name: vumi.transports.truteq.tests.test_truteq -*-
# -*- coding: utf-8 -*-

"""TruTeq USSD transport."""

from twisted.internet.defer import inlineCallbacks, maybeDeferred
from twisted.internet.protocol import Factory


from txssmi.protocol import SSMIProtocol
from txssmi import constants

from vumi import log
from vumi.components.session import SessionManager
from vumi.config import (
    ConfigText, ConfigInt, ConfigClientEndpoint, ConfigBool, ConfigDict,
    ClientEndpointFallback)
from vumi.message import TransportUserMessage
from vumi.reconnecting_client import ReconnectingClientService
from vumi.transports.base import Transport
from vumi.utils import normalize_msisdn


class TruteqTransportConfig(Transport.CONFIG_CLASS):
    username = ConfigText(
        'Username of the TruTeq account to connect to.', static=True)
    password = ConfigText(
        'Password for the TruTeq account.', static=True)
    twisted_endpoint = ConfigClientEndpoint(
        'The endpoint to connect to.',
        default='tcp:host=sms.truteq.com:port=50008', static=True,
        fallbacks=[ClientEndpointFallback()])
    link_check_period = ConfigInt(
        'Number of seconds between link checks sent to the server.',
        default=60, static=True)
    ussd_session_lifetime = ConfigInt(
        'Maximum number of seconds to retain USSD session information.',
        default=300, static=True)
    debug = ConfigBool(
        'Print verbose log output.', default=False, static=True)
    redis_manager = ConfigDict(
        'How to connect to Redis.', default={}, static=True)

    # TODO: Deprecate these fields when confmodel#5 is done.
    host = ConfigText(
        "*DEPRECATED* 'host' and 'port' fields may be used in place of the"
        " 'twisted_endpoint' field.", static=True)
    port = ConfigInt(
        "*DEPRECATED* 'host' and 'port' fields may be used in place of the"
        " 'twisted_endpoint' field.", static=True)


class TruteqTransportProtocol(SSMIProtocol):

    def connectionMade(self):
        config = self.factory.vumi_transport.get_static_config()
        self.factory.protocol_instance = self
        self.noisy = config.debug
        d = self.authenticate(config.username, config.password)
        d.addCallback(
            lambda success: (
                self.link_check.start(config.link_check_period)
                if success else self.loseConnection()))
        return d

    def connectionLost(self, reason):
        if self.link_check.running:
            self.link_check.stop()
        SSMIProtocol.connectionLost(self, reason)

    def handle_MO(self, mo):
        return self.factory.vumi_transport.handle_unhandled_message(mo)

    def handle_BINARY_MO(self, mo):
        return self.factory.vumi_transport.handle_unhandled_message(mo)

    def handle_PREMIUM_MO(self, mo):
        return self.factory.vumi_transport.handle_unhandled_message(mo)

    def handle_PREMIUM_BINARY_MO(self, mo):
        return self.factory.vumi_transport.handle_unhandled_message(mo)

    def handle_USSD_MESSAGE(self, um):
        return self.factory.vumi_transport.handle_raw_inbound_message(um)

    def handle_EXTENDED_USSD_MESSAGE(self, um):
        return self.factory.vumi_transport.handle_raw_inbound_message(um)

    def handle_LOGOUT(self, msg):
        return self.factory.vumi_transport.handle_remote_logout(msg)


class TruteqTransport(Transport):
    """
    A transport for TruTeq.

    Currently only USSD messages are supported.

    """

    CONFIG_CLASS = TruteqTransportConfig
    service_class = ReconnectingClientService
    protocol_class = TruteqTransportProtocol
    encoding = 'iso-8859-1'

    SSMI_TO_VUMI_EVENT = {
        constants.USSD_NEW: TransportUserMessage.SESSION_NEW,
        constants.USSD_RESPONSE: TransportUserMessage.SESSION_RESUME,
        constants.USSD_END: TransportUserMessage.SESSION_CLOSE,
        constants.USSD_TIMEOUT: TransportUserMessage.SESSION_CLOSE,
    }

    VUMI_TO_SSMI_EVENT = {
        TransportUserMessage.SESSION_NONE: constants.USSD_RESPONSE,
        TransportUserMessage.SESSION_NEW: constants.USSD_NEW,
        TransportUserMessage.SESSION_RESUME: constants.USSD_RESPONSE,
        TransportUserMessage.SESSION_CLOSE: constants.USSD_END,
    }

    @inlineCallbacks
    def setup_transport(self):
        config = self.get_static_config()
        self.client_factory = Factory.forProtocol(self.protocol_class)
        self.client_factory.vumi_transport = self

        prefix = "%s:ussd_codes" % (config.transport_name,)
        self.session_manager = yield SessionManager.from_redis_config(
            config.redis_manager, prefix, config.ussd_session_lifetime)
        self.client_service = self.get_service(
            config.twisted_endpoint, self.client_factory)

    def get_service(self, endpoint, factory):
        client_service = self.service_class(endpoint, factory)
        client_service.startService()
        return client_service

    def teardown_transport(self):
        d = maybeDeferred(self.client_service.stopService)
        d.addCallback(lambda _: self.session_manager.stop())
        return d

    @inlineCallbacks
    def handle_raw_inbound_message(self, ussd_message):
        if ussd_message.command_name == 'EXTENDED_USSD_MESSAGE':
            genfields = {
                'IMSI': '',
                'Subscriber Type': '',
                'OperatorID': '',
                'SessionID': '',
                'ValiPort': '',
            }
            genfield_values = ussd_message.genfields.split(':')
            genfields.update(
                dict(zip(genfields.keys(), genfield_values)))
        else:
            genfields = {}

        session_event = self.SSMI_TO_VUMI_EVENT[ussd_message.type]
        msisdn = normalize_msisdn(ussd_message.msisdn)
        message = ussd_message.message.decode(self.encoding)

        if session_event == TransportUserMessage.SESSION_NEW:
            # If it's a new session then store the message as the USSD code
            if not message.endswith('#'):
                message = '%s#' % (message,)
            session = yield self.session_manager.create_session(
                msisdn, ussd_code=message)
            text = None
        else:
            session = yield self.session_manager.load_session(msisdn)
            text = message

        if session_event == TransportUserMessage.SESSION_CLOSE:
            yield self.session_manager.clear_session(msisdn)

        yield self.publish_message(
            from_addr=msisdn,
            to_addr=session['ussd_code'],
            session_event=session_event,
            content=text,
            transport_type='ussd',
            transport_metadata={},
            helper_metadata={
                'truteq': {
                    'genfields': genfields,
                }
            })

    def handle_outbound_message(self, message):
        protocol = self.client_factory.protocol_instance
        text = message.get('content') or ''

        # Truteq uses \r as a message delimiter in the protocol.
        # Make sure we're only sending \n for new lines.
        text = '\n'.join(text.splitlines()).encode(self.encoding)

        ssmi_session_type = self.VUMI_TO_SSMI_EVENT[message['session_event']]
        # We need to send unicode data to ssmi_client, but bytes for msisdn.
        msisdn = message['to_addr'].strip('+').encode(self.encoding)
        return protocol.send_ussd_message(msisdn, text, ssmi_session_type)

    def handle_remote_logout(self, msg):
        log.warning('Received remote logout command: %r' % (
            msg,))

    def handle_unhandled_message(self, mo):
        log.warning('Received unsupported message, dropping: %r.' % (
            mo,))



# -*- encoding: utf-8 -*-

import json
from urllib import urlencode

from twisted.internet.defer import inlineCallbacks, DeferredQueue, returnValue

from vumi.utils import http_request, http_request_full
from vumi.tests.fake_connection import FakeHttpServer
from vumi.tests.helpers import VumiTestCase
from vumi.transports.cellulant import CellulantSmsTransport
from vumi.transports.tests.helpers import TransportHelper


class FakeCellulant(object):
    def __init__(self):
        self.cellulant_sms_calls = DeferredQueue()
        self.fake_http = FakeHttpServer(self.handle_request)
        self.response = ''
        self.get_agent = self.fake_http.get_agent
        self.get = self.cellulant_sms_calls.get

    def handle_request(self, request):
        self.cellulant_sms_calls.put(request)
        return self.response


class TestCellulantSmsTransport(VumiTestCase):

    @inlineCallbacks
    def setUp(self):
        self.fake_cellulant = FakeCellulant()
        self.base_url = "http://cellulant.example.com/"

        self.config = {
            'web_path': "foo",
            'web_port': 0,
            'credentials': {
                '2371234567': {
                    'username': 'user',
                    'password': 'pass',
                },
                '9292': {
                    'username': 'other-user',
                    'password': 'other-pass',
                }
            },
            'outbound_url': self.base_url,
        }
        self.tx_helper = self.add_helper(
            TransportHelper(CellulantSmsTransport))
        self.transport = yield self.tx_helper.get_transport(self.config)
        self.transport.agent_factory = self.fake_cellulant.get_agent
        self.transport_url = self.transport.get_transport_url()

    def mkurl(self, content, from_addr="2371234567", **kw):
        params = {
            'SOURCEADDR': from_addr,
            'DESTADDR': '12345',
            'MESSAGE': content,
            'ID': '1234567',
            }
        params.update(kw)
        return self.mkurl_raw(**params)

    def mkurl_raw(self, **params):
        return '%s%s?%s' % (
            self.transport_url,
            self.config['web_path'],
            urlencode(params)
        )

    @inlineCallbacks
    def test_health(self):
        result = yield http_request(
            self.transport_url + "health", "", method='GET')
        self.assertEqual(json.loads(result), {'pending_requests': 0})

    @inlineCallbacks
    def test_inbound(self):
        url = self.mkurl('hello')
        response = yield http_request(url, '', method='GET')
        [msg] = self.tx_helper.get_dispatched_inbound()
        self.assertEqual(msg['transport_name'], self.tx_helper.transport_name)
        self.assertEqual(msg['to_addr'], "12345")
        self.assertEqual(msg['from_addr'], "2371234567")
        self.assertEqual(msg['content'], "hello")
        self.assertEqual(json.loads(response),
                         {'message_id': msg['message_id']})

    @inlineCallbacks
    def test_outbound(self):
        yield self.tx_helper.make_dispatch_outbound(
            "hello world", to_addr="2371234567")
        req = yield self.fake_cellulant.get()
        self.assertEqual(req.path, self.base_url)
        self.assertEqual(req.method, 'GET')
        self.assertEqual({
            'username': ['other-user'],
            'password': ['other-pass'],
            'source': ['9292'],
            'destination': ['2371234567'],
            'message': ['hello world'],
        }, req.args)

    @inlineCallbacks
    def test_outbound_creds_selection(self):
        yield self.tx_helper.make_dispatch_outbound(
            "hello world", to_addr="2371234567", from_addr='2371234567')
        req = yield self.fake_cellulant.get()
        self.assertEqual(req.path, self.base_url)
        self.assertEqual(req.method, 'GET')
        self.assertEqual({
            'username': ['user'],
            'password': ['pass'],
            'source': ['2371234567'],
            'destination': ['2371234567'],
            'message': ['hello world'],
        }, req.args)

        yield self.tx_helper.make_dispatch_outbound(
            "hello world", to_addr="2371234567", from_addr='9292')
        req = yield self.fake_cellulant.get()
        self.assertEqual(req.path, self.base_url)
        self.assertEqual(req.method, 'GET')
        self.assertEqual({
            'username': ['other-user'],
            'password': ['other-pass'],
            'source': ['9292'],
            'destination': ['2371234567'],
            'message': ['hello world'],
        }, req.args)

    @inlineCallbacks
    def test_handle_non_ascii_input(self):
        url = self.mkurl(u"öæł".encode("utf-8"))
        response = yield http_request(url, '', method='GET')
        [msg] = self.tx_helper.get_dispatched_inbound()
        self.assertEqual(msg['transport_name'], self.tx_helper.transport_name)
        self.assertEqual(msg['to_addr'], "12345")
        self.assertEqual(msg['from_addr'], "2371234567")
        self.assertEqual(msg['content'], u"öæł")
        self.assertEqual(json.loads(response),
                         {'message_id': msg['message_id']})

    @inlineCallbacks
    def test_bad_parameter(self):
        url = self.mkurl('hello', foo='bar')
        response = yield http_request_full(url, '', method='GET')
        self.assertEqual(400, response.code)
        self.assertEqual(json.loads(response.delivered_body),
                         {'unexpected_parameter': ['foo']})

    @inlineCallbacks
    def test_missing_parameters(self):
        url = self.mkurl_raw(ID='12345678', DESTADDR='12345', MESSAGE='hello')
        response = yield http_request_full(url, '', method='GET')
        self.assertEqual(400, response.code)
        self.assertEqual(json.loads(response.delivered_body),
                         {'missing_parameter': ['SOURCEADDR']})

    @inlineCallbacks
    def test_ignored_parameters(self):
        url = self.mkurl('hello', channelID='a', keyword='b', CHANNELID='c',
                         serviceID='d', SERVICEID='e', unsub='f')
        response = yield http_request(url, '', method='GET')
        [msg] = self.tx_helper.get_dispatched_inbound()
        self.assertEqual(msg['content'], "hello")
        self.assertEqual(json.loads(response),
                         {'message_id': msg['message_id']})


class TestAcksCellulantSmsTransport(VumiTestCase):

    @inlineCallbacks
    def setUp(self):
        self.fake_cellulant = FakeCellulant()
        self.base_url = "http://cellulant.example.com/"

        self.config = {
            'web_path': "foo",
            'web_port': 0,
            'credentials': {
                '2371234567': {
                    'username': 'user',
                    'password': 'pass',
                },
                '9292': {
                    'username': 'other-user',
                    'password': 'other-pass',
                }
            },
            'outbound_url': self.base_url,
            'validation_mode': 'permissive',
        }
        self.tx_helper = self.add_helper(
            TransportHelper(CellulantSmsTransport))
        self.transport = yield self.tx_helper.get_transport(self.config)
        self.transport.agent_factory = self.fake_cellulant.get_agent
        self.transport_url = self.transport.get_transport_url()

    def mock_response(self, response):
        self.fake_cellulant.response = response

    @inlineCallbacks
    def mock_event(self, msg, nr_events):
        self.mock_response(msg)
        yield self.tx_helper.make_dispatch_outbound(
            "foo", to_addr='2371234567', message_id='id_%s' % (msg,))
        yield self.fake_cellulant.get()
        events = yield self.tx_helper.wait_for_dispatched_events(nr_events)
        returnValue(events)

    @inlineCallbacks
    def test_nack_param_error_E0(self):
        [nack] = yield self.mock_event('E0', 1)
        self.assertEqual(nack['event_type'], 'nack')
        self.assertEqual(nack['user_message_id'], 'id_E0')
        self.assertEqual(nack['nack_reason'],
                         self.transport.KNOWN_ERROR_RESPONSE_CODES['E0'])

    @inlineCallbacks
    def test_nack_login_error_E1(self):
        [nack] = yield self.mock_event('E1', 1)
        self.assertEqual(nack['event_type'], 'nack')
        self.assertEqual(nack['user_message_id'], 'id_E1')
        self.assertEqual(nack['nack_reason'],
                         self.transport.KNOWN_ERROR_RESPONSE_CODES['E1'])

    @inlineCallbacks
    def test_nack_credits_error_E2(self):
        [nack] = yield self.mock_event('E2', 1)
        self.assertEqual(nack['event_type'], 'nack')
        self.assertEqual(nack['user_message_id'], 'id_E2')
        self.assertEqual(nack['nack_reason'],
                         self.transport.KNOWN_ERROR_RESPONSE_CODES['E2'])

    @inlineCallbacks
    def test_nack_delivery_failed_1005(self):
        [nack] = yield self.mock_event('1005', 1)
        self.assertEqual(nack['event_type'], 'nack')
        self.assertEqual(nack['user_message_id'], 'id_1005')
        self.assertEqual(nack['nack_reason'],
                         self.transport.KNOWN_ERROR_RESPONSE_CODES['1005'])

    @inlineCallbacks
    def test_unknown_response(self):
        [nack] = yield self.mock_event('something_unexpected', 1)
        self.assertEqual(nack['event_type'], 'nack')
        self.assertEqual(nack['user_message_id'], 'id_something_unexpected')
        self.assertEqual(nack['nack_reason'],
                         'Unknown response code: something_unexpected')

    @inlineCallbacks
    def test_ack_success(self):
        [event] = yield self.mock_event('1', 1)
        self.assertEqual(event['event_type'], 'ack')
        self.assertEqual(event['user_message_id'], 'id_1')


class TestPermissiveCellulantSmsTransport(VumiTestCase):

    @inlineCallbacks
    def setUp(self):
        self.fake_cellulant = FakeCellulant()
        self.base_url = "http://cellulant.example.com/"

        self.config = {
            'web_path': "foo",
            'web_port': 0,
            'credentials': {
                '2371234567': {
                    'username': 'user',
                    'password': 'pass',
                },
                '9292': {
                    'username': 'other-user',
                    'password': 'other-pass',
                }
            },
            'outbound_url': self.base_url,
            'validation_mode': 'permissive',
        }
        self.tx_helper = self.add_helper(
            TransportHelper(CellulantSmsTransport))
        self.transport = yield self.tx_helper.get_transport(self.config)
        self.transport.agent_factory = self.fake_cellulant.get_agent
        self.transport_url = self.transport.get_transport_url()

    def mkurl(self, content, from_addr="2371234567", **kw):
        params = {
            'SOURCEADDR': from_addr,
            'DESTADDR': '12345',
            'MESSAGE': content,
            'ID': '1234567',
            }
        params.update(kw)
        return self.mkurl_raw(**params)

    def mkurl_raw(self, **params):
        return '%s%s?%s' % (
            self.transport_url,
            self.config['web_path'],
            urlencode(params)
        )

    @inlineCallbacks
    def test_bad_parameter_in_permissive_mode(self):
        url = self.mkurl('hello', foo='bar')
        response = yield http_request_full(url, '', method='GET')
        [msg] = self.tx_helper.get_dispatched_inbound()
        self.assertEqual(200, response.code)
        self.assertEqual(json.loads(response.delivered_body),
                         {'message_id': msg['message_id']})

    @inlineCallbacks
    def test_missing_parameters(self):
        url = self.mkurl_raw(ID='12345678', DESTADDR='12345', MESSAGE='hello')
        response = yield http_request_full(url, '', method='GET')
        self.assertEqual(400, response.code)
        self.assertEqual(json.loads(response.delivered_body),
                         {'missing_parameter': ['SOURCEADDR']})

    @inlineCallbacks
    def test_ignored_parameters(self):
        url = self.mkurl('hello', channelID='a', keyword='b', CHANNELID='c',
                         serviceID='d', SERVICEID='e', unsub='f')
        response = yield http_request(url, '', method='GET')
        [msg] = self.tx_helper.get_dispatched_inbound()
        self.assertEqual(msg['content'], "hello")
        self.assertEqual(json.loads(response),
                         {'message_id': msg['message_id']})



# -*- encoding: utf-8 -*-

"""Tests for vumi.demos.hangman."""

import string

from twisted.internet.defer import inlineCallbacks, returnValue
from twisted.internet import reactor
from twisted.web.server import Site
from twisted.web.resource import Resource
from twisted.web.static import Data

from vumi.demos.hangman import HangmanGame, HangmanWorker
from vumi.message import TransportUserMessage
from vumi.application.tests.helpers import ApplicationHelper
from vumi.tests.helpers import VumiTestCase


def mkstate(word, guesses, msg):
    return {'word': word, 'guesses': guesses, 'msg': msg}


class TestHangmanGame(VumiTestCase):
    def test_easy_game(self):
        game = HangmanGame(word='moo')
        game.event('m')
        game.event('o')
        self.assertTrue(game.won())
        self.assertEqual(
            game.state(), mkstate('moo', 'mo', 'Flawless victory!'))

    def test_incorrect_guesses(self):
        game = HangmanGame(word='moo')
        game.event('f')
        game.event('g')
        self.assertFalse(game.won())
        self.assertEqual(
            game.state(), mkstate('moo', 'fg', "Word contains no 'g'. :("))

    def test_repeated_guesses(self):
        game = HangmanGame(word='moo')
        game.event('f')
        game.event('f')
        self.assertFalse(game.won())
        self.assertEqual(
            game.state(), mkstate('moo', 'f', "You've already guessed 'f'."))

    def test_button_mashing(self):
        game = HangmanGame(word='moo')
        for event in string.lowercase.replace('o', ''):
            game.event(event)
        game.event('o')
        self.assertTrue(game.won())
        self.assertEqual(
            game.state(), mkstate('moo', string.lowercase, "Button mashing!"))

    def test_new_game(self):
        game = HangmanGame(word='moo')
        for event in ('m', 'o', '-'):
            game.event(event)
        self.assertEqual(
            game.state(), mkstate('moo', 'mo', 'Flawless victory!'))
        self.assertEqual(game.exit_code, game.DONE_WANTS_NEW)

    def test_from_state(self):
        game = HangmanGame.from_state(mkstate("bar", "xyz", "Eep?"))
        self.assertEqual(game.word, "bar")
        self.assertEqual(game.guesses, set("xyz"))
        self.assertEqual(game.msg, "Eep?")
        self.assertEqual(game.exit_code, game.NOT_DONE)

    def test_from_state_non_ascii(self):
        game = HangmanGame.from_state(
            mkstate("b\xc3\xa4r".decode("utf-8"), "xyz", "Eep?"))
        self.assertEqual(game.word, u"b\u00e4r")
        self.assertEqual(game.guesses, set("xyz"))
        self.assertEqual(game.msg, "Eep?")
        self.assertEqual(game.exit_code, game.NOT_DONE)

    def test_exit(self):
        game = HangmanGame('elephant')
        game.event('0')
        self.assertEqual(game.exit_code, game.DONE)
        self.assertEqual(game.draw_board(), "Adieu!")

    def test_draw_board(self):
        game = HangmanGame('word')
        board = game.draw_board()
        msg, word, guesses, prompt, end = board.split("\n")
        self.assertEqual(msg, "New game!")
        self.assertEqual(word, "Word: ____")
        self.assertEqual(guesses, "Letters guessed so far: ")
        self.assertEqual(prompt, "Enter next guess (0 to quit):")

    def test_draw_board_at_end_of_game(self):
        game = HangmanGame('m')
        game.event('m')
        board = game.draw_board()
        msg, word, guesses, prompt, end = board.split("\n")
        self.assertEqual(msg, "Flawless victory!")
        self.assertEqual(word, "Word: m")
        self.assertEqual(guesses, "Letters guessed so far: m")
        self.assertEqual(prompt, "Enter anything to start a new game"
                                 " (0 to quit):")

    def test_displaying_word(self):
        game = HangmanGame('word')
        game.event('w')
        game.event('r')
        board = game.draw_board()
        _msg, word, _guesses, _prompt, _end = board.split("\n")
        self.assertEqual(word, "Word: w_r_")

    def test_displaying_guesses(self):
        game = HangmanGame('word')
        game.event('w')
        board = game.draw_board()
        msg, _word, _guesses, _prompt, _end = board.split("\n")
        self.assertEqual(msg, "Word contains at least one 'w'! :D")

        game.event('w')
        board = game.draw_board()
        msg, _word, _guesses, _prompt, _end = board.split("\n")
        self.assertEqual(msg, "You've already guessed 'w'.")

        game.event('x')
        board = game.draw_board()
        msg, _word, _guesses, _prompt, _end = board.split("\n")
        self.assertEqual(msg, "Word contains no 'x'. :(")

    def test_garbage_input(self):
        game = HangmanGame(word="zoo")
        for garbage in [":", "!", "\x00", "+", "abc", ""]:
            game.event(garbage)
        self.assertEqual(game.guesses, set())
        game.event('z')
        game.event('o')
        self.assertTrue(game.won())


class TestHangmanWorker(VumiTestCase):

    @inlineCallbacks
    def setUp(self):
        root = Resource()
        # data is elephant with a UTF-8 encoded BOM
        # it is a sad elephant (as seen in the wild)
        root.putChild("word", Data('\xef\xbb\xbfelephant\r\n', 'text/html'))
        site_factory = Site(root)
        self.webserver = yield reactor.listenTCP(
            0, site_factory, interface='127.0.0.1')
        self.add_cleanup(self.webserver.loseConnection)
        addr = self.webserver.getHost()
        random_word_url = "http://%s:%s/word" % (addr.host, addr.port)

        self.app_helper = self.add_helper(ApplicationHelper(HangmanWorker))

        self.worker = yield self.app_helper.get_application({
            'worker_name': 'test_hangman',
            'random_word_url': random_word_url,
        })
        yield self.worker.session_manager.redis._purge_all()  # just in case

    def send(self, content, session_event=None):
        return self.app_helper.make_dispatch_inbound(
            content, session_event=session_event)

    @inlineCallbacks
    def recv(self, n=0):
        msgs = yield self.app_helper.wait_for_dispatched_outbound(n)

        def reply_code(msg):
            if msg['session_event'] == TransportUserMessage.SESSION_CLOSE:
                return 'end'
            return 'reply'

        returnValue([(reply_code(msg), msg['content']) for msg in msgs])

    @inlineCallbacks
    def test_new_session(self):
        yield self.send(None, TransportUserMessage.SESSION_NEW)
        replies = yield self.recv(1)
        self.assertEqual(len(replies), 1)

        reply = replies[0]
        self.assertEqual(reply[0], 'reply')
        self.assertEqual(reply[1],
                         "New game!\n"
                         "Word: ________\n"
                         "Letters guessed so far: \n"
                         "Enter next guess (0 to quit):\n")

    @inlineCallbacks
    def test_random_word(self):
        word = yield self.worker.random_word(
            self.worker.config['random_word_url'])
        self.assertEqual(word, 'elephant')

    @inlineCallbacks
    def test_full_session(self):
        yield self.send(None, TransportUserMessage.SESSION_NEW)
        for event in ('e', 'l', 'p', 'h', 'a', 'n', 'o', 't'):
            yield self.send(event, TransportUserMessage.SESSION_RESUME)

        replies = yield self.recv(9)
        self.assertEqual(len(replies), 9)

        last_reply = replies[-1]
        self.assertEqual(last_reply[0], 'reply')
        self.assertEqual(last_reply[1],
                         "Epic victory!\n"
                         "Word: elephant\n"
                         "Letters guessed so far: aehlnopt\n"
                         "Enter anything to start a new game (0 to quit):\n")

        yield self.send('1')
        replies = yield self.recv(10)
        last_reply = replies[-1]
        self.assertEqual(last_reply[0], 'reply')
        self.assertEqual(last_reply[1],
                         "New game!\n"
                         "Word: ________\n"
                         "Letters guessed so far: \n"
                         "Enter next guess (0 to quit):\n")

        yield self.send('0')
        replies = yield self.recv(11)
        last_reply = replies[-1]
        self.assertEqual(last_reply[0], 'end')
        self.assertEqual(last_reply[1], "Adieu!")

    @inlineCallbacks
    def test_close_session(self):
        yield self.send(None, TransportUserMessage.SESSION_CLOSE)
        replies = yield self.recv()
        self.assertEqual(replies, [])

    @inlineCallbacks
    def test_non_ascii_input(self):
        yield self.send(None, TransportUserMessage.SESSION_NEW)
        for event in (u'ü', u'æ'):
            yield self.send(event, TransportUserMessage.SESSION_RESUME)

        replies = yield self.recv(3)
        self.assertEqual(len(replies), 3)

        for reply in replies[1:]:
            self.assertEqual(reply[0], 'reply')
            self.assertTrue(reply[1].startswith(
                'Letters of the alphabet only please.'))



"""
Some utilities and things for testing various bits of SMPP.
"""

from twisted.internet.defer import DeferredQueue
from smpp.pdu_inspector import unpack_pdu

from vumi.transports.smpp.deprecated.clientserver.server import SmscServer


class SmscTestServer(SmscServer):
    """
    SMSC subclass that records inbound and outbound PDUs for later assertion.
    """

    def __init__(self, delivery_report_string=None):
        self.pdu_queue = DeferredQueue()
        SmscServer.__init__(self, delivery_report_string)

    def handle_data(self, data):
        self.pdu_queue.put({
                'direction': 'inbound',
                'pdu': unpack_pdu(data),
                })
        return SmscServer.handle_data(self, data)

    def send_pdu(self, pdu):
        self.pdu_queue.put({
                'direction': 'outbound',
                'pdu': pdu.get_obj(),
                })
        return SmscServer.send_pdu(self, pdu)



# -*- encoding: utf-8 -*-

"""Tests for vumi.blinkenlights.heartbeat.monitor"""

import time
import json

from twisted.internet.defer import inlineCallbacks

from vumi.blinkenlights.heartbeat import publisher
from vumi.blinkenlights.heartbeat import monitor
from vumi.blinkenlights.heartbeat.storage import issue_key
from vumi.utils import generate_worker_id
from vumi.tests.helpers import VumiTestCase, WorkerHelper, PersistenceHelper


def expected_wkr_dict():
    wkr = {
        'id': 'system-1:foo',
        'name': 'foo',
        'system_id': 'system-1',
        'min_procs': 1,
        'hosts': [{'host': 'host-1', 'proc_count': 1}],
    }
    return wkr


def expected_sys_dict():
    sys = {
        'name': 'system-1',
        'id': 'system-1',
        'timestamp': int(435),
        'workers': [expected_wkr_dict()],
    }
    return sys


class TestWorkerInstance(VumiTestCase):

    def test_create(self):
        worker = monitor.WorkerInstance('foo', 34)
        self.assertEqual(worker.hostname, 'foo')
        self.assertEqual(worker.pid, 34)

    def test_equiv(self):
        self.assertEqual(monitor.WorkerInstance('foo', 34),
                         monitor.WorkerInstance('foo', 34))
        self.failIfEqual(monitor.WorkerInstance('foo', 4),
                         monitor.WorkerInstance('foo', 34))
        self.failIfEqual(monitor.WorkerInstance('fo', 34),
                         monitor.WorkerInstance('foo', 34))

    def test_hash(self):
        worker1 = monitor.WorkerInstance('foo', 34)
        worker2 = monitor.WorkerInstance('foo', 34)
        worker3 = monitor.WorkerInstance('foo', 35)
        worker4 = monitor.WorkerInstance('bar', 34)
        self.assertEqual(hash(worker1), hash(worker2))
        self.assertNotEqual(hash(worker1), hash(worker3))
        self.assertNotEqual(hash(worker1), hash(worker4))


class TestWorker(VumiTestCase):

    def test_to_dict(self):
        wkr = monitor.Worker('system-1', 'foo', 1)
        wkr.record('host-1', 34)

        wkr.snapshot()

        obj = wkr.to_dict()
        self.assertEqual(obj, expected_wkr_dict())

    def test_compute_host_info(self):
        wkr = monitor.Worker('system-1', 'foo', 1)
        wkr.record('host-1', 34)
        wkr.record('host-1', 546)

        wkr.snapshot()

        counts = wkr._compute_host_info(wkr._instances)
        self.assertEqual(counts['host-1'], 2)

    def test_snapshot(self):
        wkr = monitor.Worker('system-1', 'foo', 1)

        wkr.record('host-1', 34)
        wkr.record('host-1', 546)

        self.assertEqual(len(wkr._instances_active), 2)
        self.assertEqual(len(wkr._instances), 0)

        wkr.snapshot()

        self.assertEqual(len(wkr._instances_active), 0)
        self.assertEqual(len(wkr._instances), 2)


class TestSystem(VumiTestCase):

    def test_to_dict(self):
        wkr = monitor.Worker('system-1', 'foo', 1)
        sys = monitor.System('system-1', 'system-1', [wkr])
        wkr.record('host-1', 34)
        wkr.snapshot()
        obj = sys.to_dict()
        obj['timestamp'] = 435
        self.assertEqual(obj, expected_sys_dict())

    def test_dumps(self):
        wkr = monitor.Worker('system-1', 'foo', 1)
        sys = monitor.System('system-1', 'system-1', [wkr])
        wkr.record('host-1', 34)
        wkr.snapshot()
        obj_json = sys.dumps()
        obj = json.loads(obj_json)
        obj['timestamp'] = 435
        self.assertEqual(obj, expected_sys_dict())


class TestHeartBeatMonitor(VumiTestCase):

    @inlineCallbacks
    def setUp(self):
        self.persistence_helper = self.add_helper(PersistenceHelper())
        self.worker_helper = self.add_helper(WorkerHelper())
        config = {
            'deadline': 30,
            'redis_manager': {
                'key_prefix': 'heartbeats',
                'db': 5,
                'FAKE_REDIS': True,
            },
            'monitored_systems': {
                'system-1': {
                    'system_name': 'system-1',
                    'system_id': 'system-1',
                    'workers': {
                        'twitter_transport': {
                            'name': 'twitter_transport',
                            'min_procs': 2,
                        }
                    }
                }
            }
        }
        self.worker = yield self.worker_helper.get_worker(
            monitor.HeartBeatMonitor, config, start=False)

    def gen_fake_attrs(self, timestamp):
        sys_id = 'system-1'
        wkr_name = 'twitter_transport'
        wkr_id = generate_worker_id(sys_id, wkr_name)
        attrs = {
            'version': publisher.HeartBeatMessage.VERSION_20130319,
            'system_id': sys_id,
            'worker_id': wkr_id,
            'worker_name': wkr_name,
            'hostname': "test-host-1",
            'timestamp': timestamp,
            'pid': 345,
        }
        return attrs

    @inlineCallbacks
    def test_update(self):
        # Test the processing of a message.

        yield self.worker.startWorker()
        attrs1 = self.gen_fake_attrs(time.time())
        attrs2 = self.gen_fake_attrs(time.time())

        # process the fake message (and process it twice to verify idempotency)
        self.worker.update(attrs1)
        self.worker.update(attrs1)

        # retrieve the instance set corresponding to the worker_id in the
        # fake message
        wkr = self.worker._workers[attrs1['worker_id']]
        self.assertEqual(len(wkr._instances_active), 1)
        inst = wkr._instances_active.pop()
        wkr._instances_active.add(inst)
        self.assertEqual(inst.hostname, "test-host-1")
        self.assertEqual(inst.pid, 345)

        # now process a message from another instance of the worker
        # and verify that there are two recorded instances
        attrs2['hostname'] = 'test-host-2'
        self.worker.update(attrs2)
        self.assertEqual(len(wkr._instances_active), 2)

    @inlineCallbacks
    def test_audit_fail(self):
        # here we test the verification of a worker who
        # who had less than min_procs check in

        yield self.worker.startWorker()
        fkredis = self.worker._redis

        attrs = self.gen_fake_attrs(time.time())
        wkr_id = attrs['worker_id']
        # process the fake message ()
        yield self.worker.update(attrs)

        wkr = self.worker._workers[attrs['worker_id']]
        wkr.snapshot()

        yield wkr.audit(self.worker._storage)

        # test that an issue was opened
        self.assertEqual(wkr.procs_count, 1)
        key = issue_key(wkr_id)
        issue = json.loads((yield fkredis.get(key)))
        self.assertEqual(issue['issue_type'], 'min-procs-fail')

    @inlineCallbacks
    def test_audit_pass(self):
        # here we test the verification of a worker who
        # who had more than min_procs check in

        yield self.worker.startWorker()
        fkredis = self.worker._redis

        attrs = self.gen_fake_attrs(time.time())
        wkr_id = attrs['worker_id']
        # process the fake message ()
        yield self.worker.update(attrs)
        attrs['pid'] = 2342
        yield self.worker.update(attrs)

        wkr = self.worker._workers[attrs['worker_id']]
        wkr.snapshot()

        yield wkr.audit(self.worker._storage)

        # verify that no issue has been opened
        self.assertEqual(wkr.procs_count, 2)
        key = issue_key(wkr_id)
        issue = yield fkredis.get(key)
        self.assertEqual(issue, None)

    @inlineCallbacks
    def test_serialize_to_redis(self):
        # This covers a lot of the serialization methods
        # as well as the _sync_to_storage() function.

        yield self.worker.startWorker()
        fkredis = self.worker._redis

        attrs = self.gen_fake_attrs(time.time())

        # process the fake message
        self.worker.update(attrs)

        yield self.worker._periodic_task()

        # this blob is what should be persisted into redis (as JSON)
        expected = {
            u'name': u'system-1',
            u'id': u'system-1',
            u'timestamp': 2,
            u'workers': [{
                    u'id': u'system-1:twitter_transport',
                    u'name': u'twitter_transport',
                    u'system_id': u'system-1',
                    u'min_procs': 2,
                    u'hosts': [{u'host': u'test-host-1', u'proc_count': 1}]
            }],
        }

        # verify that the system data was persisted correctly
        system = json.loads((yield fkredis.get('system:system-1')))
        system['timestamp'] = 2
        self.assertEqual(system, expected)



# -*- coding: utf-8 -*-

import logging

from twisted.internet.defer import inlineCallbacks, returnValue
from twisted.internet.task import Clock

from smpp.pdu_builder import DeliverSM, SubmitSMResp
from vumi.config import ConfigError
from vumi.message import TransportUserMessage
from vumi.tests.helpers import VumiTestCase
from vumi.tests.utils import LogCatcher
from vumi.transports.smpp.smpp_transport import (
    message_key, remote_message_key, multipart_info_key, sequence_number_key,
    SmppTransceiverTransport, SmppTransmitterTransport, SmppReceiverTransport,
    SmppTransceiverTransportWithOldConfig)
from vumi.transports.smpp.pdu_utils import (
    pdu_ok, short_message, command_id, seq_no, pdu_tlv, unpacked_pdu_opts)
from vumi.transports.smpp.processors import SubmitShortMessageProcessor
from vumi.transports.smpp.tests.fake_smsc import FakeSMSC
from vumi.transports.tests.helpers import TransportHelper


class TestSmppTransportConfig(VumiTestCase):
    def test_host_port_fallback(self):
        """
        Old-style 'host' and 'port' fields are still supported in configs.
        """

        def parse_config(extra_config):
            config = {
                'transport_name': 'name',
                'system_id': 'foo',
                'password': 'bar',
            }
            config.update(extra_config)
            return SmppTransceiverTransport.CONFIG_CLASS(config, static=True)

        # If we don't provide an endpoint config, we get an error.
        self.assertRaises(ConfigError, parse_config, {})

        # If we do provide an endpoint config, we get an endpoint.
        cfg = {'twisted_endpoint': 'tcp:host=example.com:port=1337'}
        self.assertNotEqual(parse_config(cfg).twisted_endpoint.connect, None)

        # If we provide host and port configs, we get an endpoint.
        cfg = {'host': 'example.com', 'port': 1337}
        self.assertNotEqual(parse_config(cfg).twisted_endpoint.connect, None)


class SmppTransportTestCase(VumiTestCase):

    DR_TEMPLATE = ("id:%s sub:... dlvrd:... submit date:200101010030"
                   " done date:200101020030 stat:DELIVRD err:... text:Meep")
    DR_MINIMAL_TEMPLATE = "id:%s stat:DELIVRD text:Meep"
    transport_class = None

    def setUp(self):
        self.clock = Clock()
        self.fake_smsc = FakeSMSC()
        self.tx_helper = self.add_helper(TransportHelper(self.transport_class))
        self.default_config = {
            'transport_name': self.tx_helper.transport_name,
            'worker_name': self.tx_helper.transport_name,
            'twisted_endpoint': self.fake_smsc.endpoint,
            'delivery_report_processor': 'vumi.transports.smpp.processors.'
                                         'DeliveryReportProcessor',
            'deliver_short_message_processor': (
                'vumi.transports.smpp.processors.'
                'DeliverShortMessageProcessor'),
            'system_id': 'foo',
            'password': 'bar',
            'deliver_short_message_processor_config': {
                'data_coding_overrides': {
                    0: 'utf-8',
                }
            }
        }

    def _get_transport_config(self, config):
        """
        This is overridden in a subclass.
        """
        cfg = self.default_config.copy()
        cfg.update(config)
        return cfg

    @inlineCallbacks
    def get_transport(self, config={}, bind=True):
        cfg = self._get_transport_config(config)
        transport = yield self.tx_helper.get_transport(cfg, start=False)
        transport.clock = self.clock
        yield transport.startWorker()
        self.clock.advance(0)
        if bind:
            yield self.fake_smsc.bind()
        returnValue(transport)


class SmppTransceiverTransportTestCase(SmppTransportTestCase):

    transport_class = SmppTransceiverTransport

    @inlineCallbacks
    def test_setup_transport(self):
        transport = yield self.get_transport(bind=False)
        protocol = yield transport.service.get_protocol()
        self.assertEqual(protocol.is_bound(), False)
        yield self.fake_smsc.bind()
        self.assertEqual(protocol.is_bound(), True)

    @inlineCallbacks
    def test_mo_sms(self):
        yield self.get_transport()
        self.fake_smsc.send_mo(
            sequence_number=1, short_message='foo', source_addr='123',
            destination_addr='456')
        deliver_sm_resp = yield self.fake_smsc.await_pdu()
        self.assertTrue(pdu_ok(deliver_sm_resp))
        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg['content'], 'foo')
        self.assertEqual(msg['from_addr'], '123')
        self.assertEqual(msg['to_addr'], '456')
        self.assertEqual(msg['transport_type'], 'sms')

    @inlineCallbacks
    def test_mo_sms_empty_sms_allowed(self):
        yield self.get_transport({
            'deliver_short_message_processor_config': {
                'allow_empty_messages': True,
            }
        })
        self.fake_smsc.send_mo(
            sequence_number=1, short_message='', source_addr='123',
            destination_addr='456')
        deliver_sm_resp = yield self.fake_smsc.await_pdu()
        self.assertTrue(pdu_ok(deliver_sm_resp))
        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg['content'], '')

    @inlineCallbacks
    def test_mo_sms_empty_sms_disallowed(self):
        yield self.get_transport()
        with LogCatcher(message=r"^(Not all parts|WARNING)") as lc:
            self.fake_smsc.send_mo(
                sequence_number=1, short_message='', source_addr='123',
                destination_addr='456')
            deliver_sm_resp = yield self.fake_smsc.await_pdu()

        self.assertFalse(pdu_ok(deliver_sm_resp))

        # check that failure to process delivery report was logged
        self.assertEqual(lc.messages(), [
            "WARNING: Not decoding `None` message with data_coding=1",
            "Not all parts of the PDU were able to be decoded. "
            "Responding with ESME_RDELIVERYFAILURE.",
        ])
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')

        inbound = self.tx_helper.get_dispatched_inbound()
        self.assertEqual(inbound, [])
        events = self.tx_helper.get_dispatched_events()
        self.assertEqual(events, [])

    @inlineCallbacks
    def test_mo_delivery_report_pdu_opt_params(self):
        """
        We always treat a message with the optional PDU params set as a
        delivery report.
        """
        transport = yield self.get_transport()
        yield transport.message_stash.set_remote_message_id('bar', 'foo')

        pdu = DeliverSM(sequence_number=1, esm_class=4)
        pdu.add_optional_parameter('receipted_message_id', 'foo')
        pdu.add_optional_parameter('message_state', 2)
        yield self.fake_smsc.handle_pdu(pdu)

        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'delivery_report')
        self.assertEqual(event['delivery_status'], 'delivered')
        self.assertEqual(event['user_message_id'], 'bar')

    @inlineCallbacks
    def test_mo_delivery_report_pdu_opt_params_esm_class_not_set(self):
        """
        We always treat a message with the optional PDU params set as a
        delivery report, even if ``esm_class`` is not set.
        """
        transport = yield self.get_transport()
        yield transport.message_stash.set_remote_message_id('bar', 'foo')

        pdu = DeliverSM(sequence_number=1)
        pdu.add_optional_parameter('receipted_message_id', 'foo')
        pdu.add_optional_parameter('message_state', 2)
        yield self.fake_smsc.handle_pdu(pdu)

        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'delivery_report')
        self.assertEqual(event['delivery_status'], 'delivered')
        self.assertEqual(event['user_message_id'], 'bar')

    @inlineCallbacks
    def test_mo_delivery_report_pdu_esm_class_not_set(self):
        """
        We treat a content-based DR as a normal message if the ``esm_class``
        flags are not set.
        """
        transport = yield self.get_transport()
        yield transport.message_stash.set_remote_message_id('bar', 'foo')

        self.fake_smsc.send_mo(
            sequence_number=1, short_message=self.DR_TEMPLATE % ('foo',),
            source_addr='123', destination_addr='456')
        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg['content'], self.DR_TEMPLATE % ('foo',))
        self.assertEqual(msg['from_addr'], '123')
        self.assertEqual(msg['to_addr'], '456')
        self.assertEqual(msg['transport_type'], 'sms')

        events = yield self.tx_helper.get_dispatched_events()
        self.assertEqual(events, [])

    @inlineCallbacks
    def test_mo_delivery_report_esm_class_with_full_content(self):
        """
        If ``esm_class`` and content are both set appropriately, we process the
        DR.
        """
        transport = yield self.get_transport()
        yield transport.message_stash.set_remote_message_id('bar', 'foo')

        self.fake_smsc.send_mo(
            sequence_number=1, short_message=self.DR_TEMPLATE % ('foo',),
            source_addr='123', destination_addr='456', esm_class=4)

        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'delivery_report')
        self.assertEqual(event['delivery_status'], 'delivered')
        self.assertEqual(event['user_message_id'], 'bar')

    @inlineCallbacks
    def test_mo_delivery_report_esm_class_with_short_status(self):
        """
        If the delivery report has a shorter status field, the default regex
        still matches.
        """
        transport = yield self.get_transport()
        yield transport.message_stash.set_remote_message_id('bar', 'foo')

        short_message = (
            "id:foo sub:... dlvrd:... submit date:200101010030"
            " done date:200101020030 stat:FAILED err:042 text:Meep")
        self.fake_smsc.send_mo(
            sequence_number=1, short_message=short_message,
            source_addr='123', destination_addr='456', esm_class=4)

        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'delivery_report')
        self.assertEqual(event['delivery_status'], 'failed')
        self.assertEqual(event['user_message_id'], 'bar')

    @inlineCallbacks
    def test_mo_delivery_report_esm_class_with_minimal_content(self):
        """
        If ``esm_class`` and content are both set appropriately, we process the
        DR even if the minimal subset of the content regex matches.
        """
        transport = yield self.get_transport()
        yield transport.message_stash.set_remote_message_id('bar', 'foo')

        self.fake_smsc.send_mo(
            sequence_number=1, source_addr='123', destination_addr='456',
            short_message=self.DR_MINIMAL_TEMPLATE % ('foo',), esm_class=4)

        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'delivery_report')
        self.assertEqual(event['delivery_status'], 'delivered')
        self.assertEqual(event['user_message_id'], 'bar')

    @inlineCallbacks
    def test_mo_delivery_report_content_with_nulls(self):
        """
        If ``esm_class`` and content are both set appropriately, we process the
        DR even if some content fields contain null values.
        """
        transport = yield self.get_transport()
        yield transport.message_stash.set_remote_message_id('bar', 'foo')

        content = (
            "id:%s sub:null dlvrd:null submit date:200101010030"
            " done date:200101020030 stat:DELIVRD err:null text:Meep")
        self.fake_smsc.send_mo(
            sequence_number=1, short_message=content % ("foo",),
            source_addr='123', destination_addr='456', esm_class=4)

        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'delivery_report')
        self.assertEqual(event['delivery_status'], 'delivered')
        self.assertEqual(event['user_message_id'], 'bar')

    @inlineCallbacks
    def test_mo_delivery_report_esm_class_with_bad_content(self):
        """
        If ``esm_class`` indicates a DR but the regex fails to match, we log a
        warning and do nothing.
        """
        transport = yield self.get_transport()
        yield transport.message_stash.set_remote_message_id('bar', 'foo')

        lc = LogCatcher(message="esm_class 4 indicates")
        with lc:
            self.fake_smsc.send_mo(
                sequence_number=1, source_addr='123', destination_addr='456',
                short_message="foo", esm_class=4)
            yield self.fake_smsc.await_pdu()

        # check that failure to process delivery report was logged
        [warning] = lc.logs
        self.assertEqual(
            warning["message"][0],
            "esm_class 4 indicates delivery report, but content does not"
            " match regex: 'foo'")
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')

        inbound = self.tx_helper.get_dispatched_inbound()
        self.assertEqual(inbound, [])
        events = self.tx_helper.get_dispatched_events()
        self.assertEqual(events, [])

    @inlineCallbacks
    def test_mo_delivery_report_esm_class_with_no_content(self):
        """
        If ``esm_class`` indicates a DR but the content is empty, we log a
        warning and do nothing.
        """
        transport = yield self.get_transport()
        yield transport.message_stash.set_remote_message_id('bar', 'foo')

        lc = LogCatcher(message="esm_class 4 indicates")
        with lc:
            self.fake_smsc.send_mo(
                sequence_number=1, source_addr='123', destination_addr='456',
                short_message=None, esm_class=4)
            yield self.fake_smsc.await_pdu()

        # check that failure to process delivery report was logged
        [warning] = lc.logs
        self.assertEqual(
            warning["message"][0],
            "esm_class 4 indicates delivery report, but content does not"
            " match regex: None")
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')

        inbound = self.tx_helper.get_dispatched_inbound()
        self.assertEqual(inbound, [])
        events = self.tx_helper.get_dispatched_events()
        self.assertEqual(events, [])

    @inlineCallbacks
    def test_mo_delivery_report_esm_disabled_with_full_content(self):
        """
        If ``esm_class`` checking is disabled and the content is set
        appropriately, we process the DR.
        """
        transport = yield self.get_transport({
            "delivery_report_processor_config": {
                "delivery_report_use_esm_class": False,
            }
        })
        yield transport.message_stash.set_remote_message_id('bar', 'foo')

        self.fake_smsc.send_mo(
            sequence_number=1, short_message=self.DR_TEMPLATE % ('foo',),
            source_addr='123', destination_addr='456', esm_class=0)

        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'delivery_report')
        self.assertEqual(event['delivery_status'], 'delivered')
        self.assertEqual(event['user_message_id'], 'bar')

    @inlineCallbacks
    def test_mo_delivery_report_esm_disabled_with_minimal_content(self):
        """
        If ``esm_class`` checking is disabled and the content is set
        appropriately, we process the DR even if the minimal subset of the
        content regex matches.
        """
        transport = yield self.get_transport({
            "delivery_report_processor_config": {
                "delivery_report_use_esm_class": False,
            }
        })
        yield transport.message_stash.set_remote_message_id('bar', 'foo')

        self.fake_smsc.send_mo(
            sequence_number=1, source_addr='123', destination_addr='456',
            short_message=self.DR_MINIMAL_TEMPLATE % ('foo',), esm_class=0)

        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'delivery_report')
        self.assertEqual(event['delivery_status'], 'delivered')
        self.assertEqual(event['user_message_id'], 'bar')

    @inlineCallbacks
    def test_mo_delivery_report_esm_disabled_content_with_nulls(self):
        """
        If ``esm_class`` checking is disabled and the content is set
        appropriately, we process the DR even if some content fields contain
        null values.
        """
        transport = yield self.get_transport({
            "delivery_report_processor_config": {
                "delivery_report_use_esm_class": False,
            }
        })
        yield transport.message_stash.set_remote_message_id('bar', 'foo')

        content = (
            "id:%s sub:null dlvrd:null submit date:200101010030"
            " done date:200101020030 stat:DELIVRD err:null text:Meep")
        self.fake_smsc.send_mo(
            sequence_number=1, short_message=content % ("foo",),
            source_addr='123', destination_addr='456', esm_class=0)

        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'delivery_report')
        self.assertEqual(event['delivery_status'], 'delivered')
        self.assertEqual(event['user_message_id'], 'bar')

    @inlineCallbacks
    def test_mo_sms_unicode(self):
        yield self.get_transport()
        self.fake_smsc.send_mo(
            sequence_number=1, short_message='Zo\xc3\xab', data_coding=0)
        deliver_sm_resp = yield self.fake_smsc.await_pdu()
        self.assertTrue(pdu_ok(deliver_sm_resp))
        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg['content'], u'Zoë')

    @inlineCallbacks
    def test_mo_sms_multipart_long(self):
        yield self.get_transport()
        content = '1' * 255

        pdu = DeliverSM(sequence_number=1)
        pdu.add_optional_parameter('message_payload', content.encode('hex'))
        self.fake_smsc.send_pdu(pdu)

        deliver_sm_resp = yield self.fake_smsc.await_pdu()
        self.assertEqual(1, seq_no(deliver_sm_resp))
        self.assertTrue(pdu_ok(deliver_sm_resp))
        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg['content'], content)

    @inlineCallbacks
    def test_mo_sms_multipart_udh(self):
        yield self.get_transport()
        deliver_sm_resps = []
        self.fake_smsc.send_mo(
            sequence_number=1, short_message="\x05\x00\x03\xff\x03\x01back")
        deliver_sm_resps.append((yield self.fake_smsc.await_pdu()))
        self.fake_smsc.send_mo(
            sequence_number=2, short_message="\x05\x00\x03\xff\x03\x02 at")
        deliver_sm_resps.append((yield self.fake_smsc.await_pdu()))
        self.fake_smsc.send_mo(
            sequence_number=3, short_message="\x05\x00\x03\xff\x03\x03 you")
        deliver_sm_resps.append((yield self.fake_smsc.await_pdu()))
        self.assertEqual([1, 2, 3], map(seq_no, deliver_sm_resps))
        self.assertTrue(all(map(pdu_ok, deliver_sm_resps)))
        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg['content'], u'back at you')

    @inlineCallbacks
    def test_mo_sms_multipart_udh_out_of_order(self):
        yield self.get_transport()
        deliver_sm_resps = []
        self.fake_smsc.send_mo(
            sequence_number=1, short_message="\x05\x00\x03\xff\x03\x01back")
        deliver_sm_resps.append((yield self.fake_smsc.await_pdu()))

        self.fake_smsc.send_mo(
            sequence_number=3, short_message="\x05\x00\x03\xff\x03\x03 you")
        deliver_sm_resps.append((yield self.fake_smsc.await_pdu()))

        self.fake_smsc.send_mo(
            sequence_number=2, short_message="\x05\x00\x03\xff\x03\x02 at")
        deliver_sm_resps.append((yield self.fake_smsc.await_pdu()))

        self.assertEqual([1, 3, 2], map(seq_no, deliver_sm_resps))
        self.assertTrue(all(map(pdu_ok, deliver_sm_resps)))
        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg['content'], u'back at you')

    @inlineCallbacks
    def test_mo_sms_multipart_sar(self):
        yield self.get_transport()
        deliver_sm_resps = []

        pdu1 = DeliverSM(sequence_number=1, short_message='back')
        pdu1.add_optional_parameter('sar_msg_ref_num', 1)
        pdu1.add_optional_parameter('sar_total_segments', 3)
        pdu1.add_optional_parameter('sar_segment_seqnum', 1)

        self.fake_smsc.send_pdu(pdu1)
        deliver_sm_resps.append((yield self.fake_smsc.await_pdu()))

        pdu2 = DeliverSM(sequence_number=2, short_message=' at')
        pdu2.add_optional_parameter('sar_msg_ref_num', 1)
        pdu2.add_optional_parameter('sar_total_segments', 3)
        pdu2.add_optional_parameter('sar_segment_seqnum', 2)

        self.fake_smsc.send_pdu(pdu2)
        deliver_sm_resps.append((yield self.fake_smsc.await_pdu()))

        pdu3 = DeliverSM(sequence_number=3, short_message=' you')
        pdu3.add_optional_parameter('sar_msg_ref_num', 1)
        pdu3.add_optional_parameter('sar_total_segments', 3)
        pdu3.add_optional_parameter('sar_segment_seqnum', 3)

        self.fake_smsc.send_pdu(pdu3)
        deliver_sm_resps.append((yield self.fake_smsc.await_pdu()))

        self.assertEqual([1, 2, 3], map(seq_no, deliver_sm_resps))
        self.assertTrue(all(map(pdu_ok, deliver_sm_resps)))
        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg['content'], u'back at you')

    @inlineCallbacks
    def test_mo_bad_encoding(self):
        yield self.get_transport()

        bad_pdu = DeliverSM(555,
                            short_message="SMS from server containing \xa7",
                            destination_addr="2772222222",
                            source_addr="2772000000",
                            data_coding=1)

        good_pdu = DeliverSM(555,
                             short_message="Next message",
                             destination_addr="2772222222",
                             source_addr="2772000000",
                             data_coding=1)

        yield self.fake_smsc.handle_pdu(bad_pdu)
        yield self.fake_smsc.handle_pdu(good_pdu)
        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)

        self.assertEqual(msg['message_type'], 'user_message')
        self.assertEqual(msg['transport_name'], self.tx_helper.transport_name)
        self.assertEqual(msg['content'], "Next message")

        dispatched_failures = self.tx_helper.get_dispatched_failures()
        self.assertEqual(dispatched_failures, [])

        [failure] = self.flushLoggedErrors(UnicodeDecodeError)
        message = failure.getErrorMessage()
        codec, rest = message.split(' ', 1)
        self.assertEqual(codec, "'ascii'")
        self.assertTrue(
            rest.startswith("codec can't decode byte 0xa7 in position 27"))

    @inlineCallbacks
    def test_mo_sms_failed_remote_id_lookup(self):
        yield self.get_transport()

        lc = LogCatcher(message="Failed to retrieve message id")
        with lc:
            yield self.fake_smsc.handle_pdu(
                DeliverSM(sequence_number=1, esm_class=4,
                          short_message=self.DR_TEMPLATE % ('foo',)))

        # check that failure to send delivery report was logged
        [warning] = lc.logs
        expected_msg = (
            "Failed to retrieve message id for delivery report. Delivery"
            " report from %s discarded.") % (self.tx_helper.transport_name,)
        self.assertEqual(warning['message'], (expected_msg,))
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')

    @inlineCallbacks
    def test_mt_sms(self):
        yield self.get_transport()
        msg = self.tx_helper.make_outbound('hello world')
        yield self.tx_helper.dispatch_outbound(msg)
        pdu = yield self.fake_smsc.await_pdu()
        self.assertEqual(command_id(pdu), 'submit_sm')
        self.assertEqual(short_message(pdu), 'hello world')

    @inlineCallbacks
    def test_mt_sms_bad_to_addr(self):
        yield self.get_transport()
        msg = yield self.tx_helper.make_dispatch_outbound(
            'hello world', to_addr=u'+\u2000')
        [event] = self.tx_helper.get_dispatched_events()
        self.assertEqual(event['event_type'], 'nack')
        self.assertEqual(event['user_message_id'], msg['message_id'])
        self.assertEqual(event['nack_reason'], u'Invalid to_addr: +\u2000')

    @inlineCallbacks
    def test_mt_sms_bad_from_addr(self):
        yield self.get_transport()
        msg = yield self.tx_helper.make_dispatch_outbound(
            'hello world', from_addr=u'+\u2000')
        [event] = self.tx_helper.get_dispatched_events()
        self.assertEqual(event['event_type'], 'nack')
        self.assertEqual(event['user_message_id'], msg['message_id'])
        self.assertEqual(event['nack_reason'], u'Invalid from_addr: +\u2000')

    @inlineCallbacks
    def test_mt_sms_submit_sm_encoding(self):
        yield self.get_transport({
            'submit_short_message_processor_config': {
                'submit_sm_encoding': 'latin1',
            }
        })
        yield self.tx_helper.make_dispatch_outbound(u'Zoë destroyer of Ascii!')
        submit_sm_pdu = yield self.fake_smsc.await_pdu()
        self.assertEqual(
            short_message(submit_sm_pdu),
            u'Zoë destroyer of Ascii!'.encode('latin-1'))

    @inlineCallbacks
    def test_mt_sms_submit_sm_null_message(self):
        """
        We can successfully send a message with null content.
        """
        yield self.get_transport()
        msg = self.tx_helper.make_outbound(None)
        yield self.tx_helper.dispatch_outbound(msg)
        pdu = yield self.fake_smsc.await_pdu()
        self.assertEqual(command_id(pdu), 'submit_sm')
        self.assertEqual(short_message(pdu), None)

    @inlineCallbacks
    def test_submit_sm_data_coding(self):
        yield self.get_transport({
            'submit_short_message_processor_config': {
                'submit_sm_data_coding': 8
            }
        })
        yield self.tx_helper.make_dispatch_outbound("hello world")
        submit_sm_pdu = yield self.fake_smsc.await_pdu()
        params = submit_sm_pdu['body']['mandatory_parameters']
        self.assertEqual(params['data_coding'], 8)

    @inlineCallbacks
    def test_mt_sms_ack(self):
        yield self.get_transport()
        msg = self.tx_helper.make_outbound('hello world')
        yield self.tx_helper.dispatch_outbound(msg)
        submit_sm_pdu = yield self.fake_smsc.await_pdu()
        self.fake_smsc.send_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm_pdu),
                         message_id='foo'))
        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'ack')
        self.assertEqual(event['user_message_id'], msg['message_id'])
        self.assertEqual(event['sent_message_id'], 'foo')

    @inlineCallbacks
    def test_mt_sms_disabled_ack(self):
        yield self.get_transport({'disable_ack': True})
        msg = self.tx_helper.make_outbound('hello world')
        yield self.tx_helper.dispatch_outbound(msg)
        submit_sm_pdu = yield self.fake_smsc.await_pdu()
        self.fake_smsc.send_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm_pdu),
                         message_id='foo'))
        # NOTE: We can't test for the absence of an event in isolation but we
        #       can test that for the presence of a second event only.
        fail_msg = self.tx_helper.make_outbound('hello fail')
        yield self.tx_helper.dispatch_outbound(fail_msg)
        submit_sm_fail_pdu = yield self.fake_smsc.await_pdu()
        self.fake_smsc.send_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm_fail_pdu),
                         message_id='foo', command_status='ESME_RINVDSTADR'))
        [fail] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(fail['event_type'], 'nack')

    @inlineCallbacks
    def test_mt_sms_nack(self):
        yield self.get_transport()
        msg = self.tx_helper.make_outbound('hello world')
        yield self.tx_helper.dispatch_outbound(msg)
        submit_sm_pdu = yield self.fake_smsc.await_pdu()
        self.fake_smsc.send_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm_pdu),
                         message_id='foo', command_status='ESME_RINVDSTADR'))
        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'nack')
        self.assertEqual(event['user_message_id'], msg['message_id'])
        self.assertEqual(event['nack_reason'], 'ESME_RINVDSTADR')

    @inlineCallbacks
    def test_mt_sms_failure(self):
        yield self.get_transport()
        message = yield self.tx_helper.make_dispatch_outbound(
            "message", message_id='446')
        submit_sm = yield self.fake_smsc.await_pdu()
        response = SubmitSMResp(seq_no(submit_sm), "3rd_party_id_3",
                                command_status="ESME_RSUBMITFAIL")
        # A failure PDU might not have a body.
        response.obj.pop('body')
        self.fake_smsc.send_pdu(response)

        # There should be a nack
        [nack] = yield self.tx_helper.wait_for_dispatched_events(1)

        [failure] = yield self.tx_helper.get_dispatched_failures()
        self.assertEqual(failure['reason'], 'ESME_RSUBMITFAIL')
        self.assertEqual(failure['message'], message.payload)

    @inlineCallbacks
    def test_mt_sms_failure_with_no_reason(self):

        yield self.get_transport()
        message = yield self.tx_helper.make_dispatch_outbound(
            "message", message_id='446')
        submit_sm = yield self.fake_smsc.await_pdu()

        yield self.fake_smsc.handle_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm),
                         message_id='foo',
                         command_status=None))

        # There should be a nack
        [nack] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(nack['user_message_id'], message['message_id'])
        self.assertEqual(nack['nack_reason'], 'Unspecified')

        [failure] = yield self.tx_helper.get_dispatched_failures()
        self.assertEqual(failure['reason'], 'Unspecified')

    @inlineCallbacks
    def test_mt_sms_seq_num_lookup_failure(self):
        transport = yield self.get_transport()

        lc = LogCatcher(message="Failed to retrieve message id")
        with lc:
            yield self.fake_smsc.handle_pdu(
                SubmitSMResp(sequence_number=0xbad, message_id='bad'))

        # Make sure we didn't store 'None' in redis.
        message_stash = transport.message_stash
        message_id = yield message_stash.get_internal_message_id('bad')
        self.assertEqual(message_id, None)

        # check that failure to send ack/nack was logged
        [warning] = lc.logs
        expected_msg = (
            "Failed to retrieve message id for deliver_sm_resp. ack/nack"
            " from %s discarded.") % (self.tx_helper.transport_name,)
        self.assertEqual(warning['message'], (expected_msg,))
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')

    @inlineCallbacks
    def test_mt_sms_throttled(self):
        transport = yield self.get_transport()
        transport_config = transport.get_static_config()
        msg = self.tx_helper.make_outbound('hello world')

        yield self.tx_helper.dispatch_outbound(msg)
        submit_sm_pdu = yield self.fake_smsc.await_pdu()
        with LogCatcher(message="Throttling outbound messages.") as lc:
            yield self.fake_smsc.handle_pdu(
                SubmitSMResp(sequence_number=seq_no(submit_sm_pdu),
                             message_id='foo',
                             command_status='ESME_RTHROTTLED'))
        [logmsg] = lc.logs
        self.assertEqual(logmsg['logLevel'], logging.INFO)
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')

        self.clock.advance(transport_config.throttle_delay)

        submit_sm_pdu_retry = yield self.fake_smsc.await_pdu()
        yield self.fake_smsc.handle_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm_pdu_retry),
                         message_id='bar',
                         command_status='ESME_ROK'))

        self.assertTrue(seq_no(submit_sm_pdu_retry) > seq_no(submit_sm_pdu))
        self.assertEqual(short_message(submit_sm_pdu), 'hello world')
        self.assertEqual(short_message(submit_sm_pdu_retry), 'hello world')
        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'ack')
        self.assertEqual(event['user_message_id'], msg['message_id'])

        # We're still throttled until our next attempt to unthrottle finds no
        # messages to retry. After a non-throttle submit_sm_resp, that happens
        # with no delay.
        with LogCatcher(message="No longer throttling outbound") as lc:
            self.clock.advance(0)
        [logmsg] = lc.logs
        self.assertEqual(logmsg['logLevel'], logging.INFO)
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')

    @inlineCallbacks
    def test_mt_sms_multipart_throttled(self):
        """
        When parts of a multipart message are throttled, we retry only those
        PDUs.
        """
        transport = yield self.get_transport({
            'submit_short_message_processor_config': {
                'send_multipart_udh': True,
            }
        })
        transport_config = transport.get_static_config()
        msg = self.tx_helper.make_outbound('a' * 350)  # Three parts.

        yield self.tx_helper.dispatch_outbound(msg)
        [pdu1, pdu2, pdu3] = yield self.fake_smsc.await_pdus(3)
        self.assertEqual(short_message(pdu1)[4:6], "\x03\x01")
        self.assertEqual(short_message(pdu2)[4:6], "\x03\x02")
        self.assertEqual(short_message(pdu3)[4:6], "\x03\x03")
        # Let two parts through.
        yield self.fake_smsc.submit_sm_resp(pdu1)
        yield self.fake_smsc.submit_sm_resp(pdu2)
        self.assertEqual(transport.throttled, False)
        # Throttle the third part.
        yield self.fake_smsc.submit_sm_resp(
            pdu3, command_status='ESME_RTHROTTLED')
        self.assertEqual(transport.throttled, True)

        self.clock.advance(transport_config.throttle_delay)
        retry_pdu = yield self.fake_smsc.await_pdu()
        # Assume nothing else is incrementing seuqnce numbers.
        self.assertEqual(seq_no(retry_pdu), seq_no(pdu3) + 1)
        # The retry should be identical to pdu3 except for the sequence number.
        pdu3_retry = dict((k, v.copy()) for k, v in pdu3.iteritems())
        pdu3_retry['header']['sequence_number'] = seq_no(retry_pdu)
        self.assertEqual(retry_pdu, pdu3_retry)

        # Let the retry through.
        yield self.fake_smsc.submit_sm_resp(retry_pdu)
        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'ack')
        self.assertEqual(event['user_message_id'], msg['message_id'])
        self.assertEqual(transport.throttled, True)
        # Prod the clock to notice there are no more retries and unthrottle.
        self.clock.advance(0)
        self.assertEqual(transport.throttled, False)

    @inlineCallbacks
    def test_mt_sms_throttle_while_throttled(self):
        transport = yield self.get_transport()
        transport_config = transport.get_static_config()
        msg1 = self.tx_helper.make_outbound('hello world 1')
        msg2 = self.tx_helper.make_outbound('hello world 2')

        yield self.tx_helper.dispatch_outbound(msg1)
        yield self.tx_helper.dispatch_outbound(msg2)
        [ssm_pdu1, ssm_pdu2] = yield self.fake_smsc.await_pdus(2)
        yield self.fake_smsc.handle_pdu(
            SubmitSMResp(sequence_number=seq_no(ssm_pdu1),
                         message_id='foo1', command_status='ESME_RTHROTTLED'))
        yield self.fake_smsc.handle_pdu(
            SubmitSMResp(sequence_number=seq_no(ssm_pdu2),
                         message_id='foo2', command_status='ESME_RTHROTTLED'))

        # Advance clock, still throttled.
        self.clock.advance(transport_config.throttle_delay)
        ssm_pdu1_retry1 = yield self.fake_smsc.await_pdu()
        yield self.fake_smsc.handle_pdu(
            SubmitSMResp(sequence_number=seq_no(ssm_pdu1_retry1),
                         message_id='bar1',
                         command_status='ESME_RTHROTTLED'))

        # Advance clock, message no longer throttled.
        self.clock.advance(transport_config.throttle_delay)
        ssm_pdu2_retry1 = yield self.fake_smsc.await_pdu()
        yield self.fake_smsc.handle_pdu(
            SubmitSMResp(sequence_number=seq_no(ssm_pdu2_retry1),
                         message_id='bar2',
                         command_status='ESME_ROK'))

        # Prod clock, message no longer throttled.
        self.clock.advance(0)
        ssm_pdu1_retry2 = yield self.fake_smsc.await_pdu()
        yield self.fake_smsc.handle_pdu(
            SubmitSMResp(sequence_number=seq_no(ssm_pdu1_retry2),
                         message_id='baz1',
                         command_status='ESME_ROK'))

        self.assertEqual(short_message(ssm_pdu1), 'hello world 1')
        self.assertEqual(short_message(ssm_pdu2), 'hello world 2')
        self.assertEqual(short_message(ssm_pdu1_retry1), 'hello world 1')
        self.assertEqual(short_message(ssm_pdu2_retry1), 'hello world 2')
        self.assertEqual(short_message(ssm_pdu1_retry2), 'hello world 1')
        [event2, event1] = yield self.tx_helper.wait_for_dispatched_events(2)
        self.assertEqual(event1['event_type'], 'ack')
        self.assertEqual(event1['user_message_id'], msg1['message_id'])
        self.assertEqual(event2['event_type'], 'ack')
        self.assertEqual(event2['user_message_id'], msg2['message_id'])

    @inlineCallbacks
    def test_mt_sms_reconnect_while_throttled(self):
        """
        If we reconnect while throttled, we don't try to unthrottle before the
        connection is in a suitable state.
        """
        transport = yield self.get_transport(bind=False)
        yield self.fake_smsc.bind()
        transport_config = transport.get_static_config()
        msg = self.tx_helper.make_outbound('hello world')
        yield self.tx_helper.dispatch_outbound(msg)
        ssm_pdu = yield self.fake_smsc.await_pdu()
        yield self.fake_smsc.handle_pdu(
            SubmitSMResp(sequence_number=seq_no(ssm_pdu),
                         message_id='foo1',
                         command_status='ESME_RTHROTTLED'))

        # Drop SMPP connection and check throttling.
        yield self.fake_smsc.disconnect()
        with LogCatcher(message="Can't check throttling while unbound") as lc:
            self.clock.advance(transport_config.throttle_delay)
        [logmsg] = lc.logs
        self.assertEqual(
            logmsg["message"][0],
            "Can't check throttling while unbound, trying later.")
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')

        # Fast-forward to reconnect (but don't bind) and check throttling.
        self.clock.advance(transport.service.delay)
        bind_pdu = yield self.fake_smsc.await_pdu()
        self.assertTrue(
            bind_pdu["header"]["command_id"].startswith("bind_"))
        with LogCatcher(message="Can't check throttling while unbound") as lc:
            self.clock.advance(transport_config.throttle_delay)
        [logmsg] = lc.logs
        self.assertEqual(
            logmsg["message"][0],
            "Can't check throttling while unbound, trying later.")
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')

        # Bind and check throttling.
        yield self.fake_smsc.bind(bind_pdu)
        with LogCatcher(message="Can't check throttling while unbound") as lc:
            self.clock.advance(transport_config.throttle_delay)
        self.assertEqual(lc.logs, [])

        ssm_pdu_retry = yield self.fake_smsc.await_pdu()
        yield self.fake_smsc.handle_pdu(
            SubmitSMResp(sequence_number=seq_no(ssm_pdu_retry),
                         message_id='foo',
                         command_status='ESME_ROK'))
        self.assertEqual(short_message(ssm_pdu), 'hello world')
        self.assertEqual(short_message(ssm_pdu_retry), 'hello world')
        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'ack')
        self.assertEqual(event['user_message_id'], msg['message_id'])

    @inlineCallbacks
    def test_mt_sms_tps_limits(self):
        transport = yield self.get_transport({'mt_tps': 2})

        with LogCatcher(message="Throttling outbound messages.") as lc:
            yield self.tx_helper.make_dispatch_outbound('hello world 1')
            yield self.tx_helper.make_dispatch_outbound('hello world 2')
            msg3_d = self.tx_helper.make_dispatch_outbound('hello world 3')
        [logmsg] = lc.logs
        self.assertEqual(logmsg['logLevel'], logging.INFO)
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')

        self.assertTrue(transport.throttled)
        [submit_sm_pdu1, submit_sm_pdu2] = yield self.fake_smsc.await_pdus(2)
        self.assertEqual(short_message(submit_sm_pdu1), 'hello world 1')
        self.assertEqual(short_message(submit_sm_pdu2), 'hello world 2')
        self.assertNoResult(msg3_d)

        with LogCatcher(message="No longer throttling outbound") as lc:
            self.clock.advance(1)
        [logmsg] = lc.logs
        self.assertEqual(logmsg['logLevel'], logging.INFO)
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')

        self.assertFalse(transport.throttled)
        yield msg3_d
        submit_sm_pdu3 = yield self.fake_smsc.await_pdu()
        self.assertEqual(short_message(submit_sm_pdu3), 'hello world 3')

    @inlineCallbacks
    def test_mt_sms_tps_limits_multipart(self):
        """
        TPS throttling counts PDUs, but finishes sending the current message.
        """
        transport = yield self.get_transport({
            'mt_tps': 3,
            'submit_short_message_processor_config': {
                'send_multipart_udh': True,
            },
        })
        self.assertEqual(transport.throttled, False)

        with LogCatcher(message="Throttling outbound messages.") as lc:
            yield self.tx_helper.make_dispatch_outbound('1' * 200 + 'a')
            yield self.tx_helper.make_dispatch_outbound('2' * 200 + 'b')
            msg3_d = self.tx_helper.make_dispatch_outbound('3' * 200 + 'c')
        [logmsg] = lc.logs
        self.assertEqual(logmsg['logLevel'], logging.INFO)
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')
        self.assertEqual(transport.throttled, True)
        [pdu1_1, pdu1_2, pdu2_1, pdu2_2] = yield self.fake_smsc.await_pdus(4)
        self.assertEqual(short_message(pdu1_1)[-5:], '11111')
        self.assertEqual(short_message(pdu1_2)[-5:], '1111a')
        self.assertEqual(short_message(pdu2_1)[-5:], '22222')
        self.assertEqual(short_message(pdu2_2)[-5:], '2222b')
        self.assertNoResult(msg3_d)

        with LogCatcher(message="No longer throttling outbound") as lc:
            self.clock.advance(1)
        [logmsg] = lc.logs
        self.assertEqual(logmsg['logLevel'], logging.INFO)
        self.assertEqual(transport.throttled, False)
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')

        yield msg3_d
        [pdu3_1, pdu3_2] = yield self.fake_smsc.await_pdus(2)
        self.assertEqual(short_message(pdu3_1)[-5:], '33333')
        self.assertEqual(short_message(pdu3_2)[-5:], '3333c')

    @inlineCallbacks
    def test_mt_sms_reconnect_while_tps_throttled(self):
        """
        If we reconnect while throttled due to the tps limit, we don't try to
        unthrottle before the connection is in a suitable state.
        """
        transport = yield self.get_transport({'mt_tps': 2})

        with LogCatcher(message="Throttling outbound messages.") as lc:
            yield self.tx_helper.make_dispatch_outbound('hello world 1')
            yield self.tx_helper.make_dispatch_outbound('hello world 2')
            msg3_d = self.tx_helper.make_dispatch_outbound('hello world 3')
        [logmsg] = lc.logs
        self.assertEqual(logmsg['logLevel'], logging.INFO)
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')

        self.assertTrue(transport.throttled)
        [submit_sm_pdu1, submit_sm_pdu2] = yield self.fake_smsc.await_pdus(2)
        self.assertEqual(short_message(submit_sm_pdu1), 'hello world 1')
        self.assertEqual(short_message(submit_sm_pdu2), 'hello world 2')
        self.assertNoResult(msg3_d)

        # Drop SMPP connection and check throttling.
        yield self.fake_smsc.disconnect()
        with LogCatcher(message="Can't stop throttling while unbound") as lc:
            self.clock.advance(1)
        [logmsg] = lc.logs
        self.assertEqual(logmsg['logLevel'], logging.INFO)
        self.assertTrue(transport.throttled)
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')

        # Fast-forward to reconnect (but don't bind) and check throttling.
        self.clock.advance(transport.service.delay)
        bind_pdu = yield self.fake_smsc.await_pdu()
        self.assertTrue(
            bind_pdu["header"]["command_id"].startswith("bind_"))
        with LogCatcher(message="Can't stop throttling while unbound") as lc:
            self.clock.advance(1)
        [logmsg] = lc.logs
        self.assertEqual(logmsg['logLevel'], logging.INFO)
        self.assertTrue(transport.throttled)
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')

        # Bind and check throttling.
        yield self.fake_smsc.bind(bind_pdu)
        with LogCatcher(message="No longer throttling outbound") as lc:
            self.clock.advance(1)
        [logmsg] = lc.logs
        self.assertEqual(logmsg['logLevel'], logging.INFO)
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')

        self.assertFalse(transport.throttled)
        submit_sm_pdu2 = yield self.fake_smsc.await_pdu()
        self.assertEqual(short_message(submit_sm_pdu2), 'hello world 3')

    @inlineCallbacks
    def test_mt_sms_queue_full(self):
        transport = yield self.get_transport()
        transport_config = transport.get_static_config()
        msg = self.tx_helper.make_outbound('hello world')
        yield self.tx_helper.dispatch_outbound(msg)
        submit_sm_pdu = yield self.fake_smsc.await_pdu()
        yield self.fake_smsc.handle_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm_pdu),
                         message_id='foo',
                         command_status='ESME_RMSGQFUL'))

        self.clock.advance(transport_config.throttle_delay)

        submit_sm_pdu_retry = yield self.fake_smsc.await_pdu()
        yield self.fake_smsc.handle_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm_pdu_retry),
                         message_id='bar',
                         command_status='ESME_ROK'))

        self.assertTrue(seq_no(submit_sm_pdu_retry) > seq_no(submit_sm_pdu))
        self.assertEqual(short_message(submit_sm_pdu), 'hello world')
        self.assertEqual(short_message(submit_sm_pdu_retry), 'hello world')
        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'ack')
        self.assertEqual(event['user_message_id'], msg['message_id'])

    @inlineCallbacks
    def test_mt_sms_remote_id_stored_only_on_rok(self):
        transport = yield self.get_transport()

        yield self.tx_helper.make_dispatch_outbound("msg1")
        submit_sm1 = yield self.fake_smsc.await_pdu()
        response = SubmitSMResp(
            seq_no(submit_sm1), "remote_1", command_status="ESME_RSUBMITFAIL")
        self.fake_smsc.send_pdu(response)

        yield self.tx_helper.make_dispatch_outbound("msg2")
        submit_sm2 = yield self.fake_smsc.await_pdu()
        response = SubmitSMResp(
            seq_no(submit_sm2), "remote_2", command_status="ESME_ROK")
        self.fake_smsc.send_pdu(response)

        yield self.tx_helper.wait_for_dispatched_events(2)

        self.assertFalse(
            (yield transport.redis.exists(remote_message_key('remote_1'))))

        self.assertTrue(
            (yield transport.redis.exists(remote_message_key('remote_2'))))

    @inlineCallbacks
    def test_mt_sms_unicode(self):
        yield self.get_transport()
        msg = self.tx_helper.make_outbound(u'Zoë')
        yield self.tx_helper.dispatch_outbound(msg)
        pdu = yield self.fake_smsc.await_pdu()
        self.assertEqual(command_id(pdu), 'submit_sm')
        self.assertEqual(short_message(pdu), 'Zo\xc3\xab')

    @inlineCallbacks
    def test_mt_sms_multipart_long(self):
        yield self.get_transport({
            'submit_short_message_processor_config': {
                'send_long_messages': True,
            }
        })
        # SMPP specifies that messages longer than 254 bytes should
        # be put in the message_payload field using TLVs
        content = '1' * 255
        msg = self.tx_helper.make_outbound(content)
        yield self.tx_helper.dispatch_outbound(msg)
        submit_sm = yield self.fake_smsc.await_pdu()
        self.assertEqual(pdu_tlv(submit_sm, 'message_payload').decode('hex'),
                         content)

    @inlineCallbacks
    def test_mt_sms_multipart_udh(self):
        """
        Sufficiently long messages are split into multiple PDUs with a UDH at
        the front of each.
        """
        transport = yield self.get_transport({
            'submit_short_message_processor_config': {
                'send_multipart_udh': True,
            }
        })
        content = '1' * 161
        msg = self.tx_helper.make_outbound(content)
        yield self.tx_helper.dispatch_outbound(msg)
        [submit_sm1, submit_sm2] = yield self.fake_smsc.await_pdus(2)

        self.assertEqual(
            submit_sm1["body"]["mandatory_parameters"]["esm_class"], 0x40)
        self.assertEqual(
            submit_sm2["body"]["mandatory_parameters"]["esm_class"], 0x40)

        udh_hlen, udh_tag, udh_len, udh_ref, udh_tot, udh_seq = [
            ord(octet) for octet in short_message(submit_sm1)[:6]]
        self.assertEqual(5, udh_hlen)
        self.assertEqual(0, udh_tag)
        self.assertEqual(3, udh_len)
        self.assertEqual(udh_tot, 2)
        self.assertEqual(udh_seq, 1)

        _, _, _, ref_to_udh_ref, _, udh_seq = [
            ord(octet) for octet in short_message(submit_sm2)[:6]]
        self.assertEqual(ref_to_udh_ref, udh_ref)
        self.assertEqual(udh_seq, 2)

        # Our multipart_info Redis hash should contain the number of parts and
        # have an appropriate TTL.
        mstash = transport.message_stash
        multipart_info = yield mstash.get_multipart_info(msg['message_id'])
        self.assertEqual(multipart_info, {"parts": "2"})
        mpi_ttl = yield mstash.redis.ttl(multipart_info_key(msg['message_id']))
        self.assertTrue(
            mpi_ttl <= mstash.config.submit_sm_expiry,
            "mpi_ttl (%s) > submit_sm_expiry (%s)" % (
                mpi_ttl, mstash.config.submit_sm_expiry))

    @inlineCallbacks
    def test_mt_sms_multipart_udh_one_part(self):
        """
        Messages that fit in a single part should not have a UDH.
        """
        yield self.get_transport({
            'submit_short_message_processor_config': {
                'send_multipart_udh': True,
            }
        })
        content = "1" * 158
        msg = self.tx_helper.make_outbound(content)
        yield self.tx_helper.dispatch_outbound(msg)
        submit_sm = yield self.fake_smsc.await_pdu()

        self.assertEqual(
            submit_sm["body"]["mandatory_parameters"]["esm_class"], 0)
        self.assertEqual(short_message(submit_sm), "1" * 158)

    @inlineCallbacks
    def test_mt_sms_multipart_sar(self):
        yield self.get_transport({
            'submit_short_message_processor_config': {
                'send_multipart_sar': True,
            }
        })
        content = '1' * 161
        msg = self.tx_helper.make_outbound(content)
        yield self.tx_helper.dispatch_outbound(msg)
        [submit_sm1, submit_sm2] = yield self.fake_smsc.await_pdus(2)

        ref_num = pdu_tlv(submit_sm1, 'sar_msg_ref_num')
        self.assertEqual(pdu_tlv(submit_sm1, 'sar_total_segments'), 2)
        self.assertEqual(pdu_tlv(submit_sm1, 'sar_segment_seqnum'), 1)

        self.assertEqual(pdu_tlv(submit_sm2, 'sar_msg_ref_num'), ref_num)
        self.assertEqual(pdu_tlv(submit_sm2, 'sar_total_segments'), 2)
        self.assertEqual(pdu_tlv(submit_sm2, 'sar_segment_seqnum'), 2)

    @inlineCallbacks
    def test_mt_sms_multipart_sar_one_part(self):
        """
        Messages that fit in a single part should not have SAR params set.
        """
        yield self.get_transport({
            'submit_short_message_processor_config': {
                'send_multipart_sar': True,
            }
        })
        content = '1' * 158
        msg = self.tx_helper.make_outbound(content)
        yield self.tx_helper.dispatch_outbound(msg)
        submit_sm = yield self.fake_smsc.await_pdu()

        self.assertEqual(unpacked_pdu_opts(submit_sm), {})
        self.assertEqual(short_message(submit_sm), "1" * 158)

    @inlineCallbacks
    def test_mt_sms_multipart_ack(self):
        """
        When all PDUs of a multipart message have been successfully
        acknowledged, we clean up the relevant transient state and send an ack.
        """
        transport = yield self.get_transport({
            'submit_short_message_processor_config': {
                'send_multipart_udh': True,
            }
        })
        content = '1' * 161
        msg = self.tx_helper.make_outbound(content)
        yield self.tx_helper.dispatch_outbound(msg)
        [submit_sm1, submit_sm2] = yield self.fake_smsc.await_pdus(2)

        # Our multipart_info Redis hash should contain the number of parts and
        # have an appropriate TTL.
        mstash = transport.message_stash
        multipart_info = yield mstash.get_multipart_info(msg['message_id'])
        self.assertEqual(multipart_info, {"parts": "2"})
        mpi_ttl = yield mstash.redis.ttl(multipart_info_key(msg['message_id']))
        self.assertTrue(
            mpi_ttl <= mstash.config.submit_sm_expiry,
            "mpi_ttl (%s) > submit_sm_expiry (%s)" % (
                mpi_ttl, mstash.config.submit_sm_expiry))

        # We get one response per PDU, so we only send the ack after receiving
        # both responses.
        self.fake_smsc.send_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm1), message_id='foo'))
        self.fake_smsc.send_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm2), message_id='bar'))
        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'ack')
        self.assertEqual(event['user_message_id'], msg['message_id'])
        self.assertEqual(event['sent_message_id'], 'bar,foo')

        # After all parts are acknowledged, our multipart_info hash should have
        # the details of the responses and a much shorter TTL.
        mstash = transport.message_stash
        multipart_info = yield mstash.get_multipart_info(msg['message_id'])
        self.assertEqual(multipart_info, {
            "parts": "2",
            "event_counter": "2",
            "part:foo": "ack",
            "part:bar": "ack",
        })
        mpi_ttl = yield mstash.redis.ttl(multipart_info_key(msg['message_id']))
        self.assertTrue(
            mpi_ttl <= mstash.config.completed_multipart_info_expiry,
            "mpi_ttl (%s) > completed_multipart_info_expiry (%s)" % (
                mpi_ttl, mstash.config.completed_multipart_info_expiry))

    @inlineCallbacks
    def test_mt_sms_multipart_fail_first_part(self):
        """
        When all PDUs of a multipart message have been acknowledged and at
        least one of them failed, we clean up the relevant transient state and
        send a nack.
        """
        transport = yield self.get_transport({
            'submit_short_message_processor_config': {
                'send_multipart_udh': True,
            }
        })
        content = '1' * 161
        msg = self.tx_helper.make_outbound(content)
        yield self.tx_helper.dispatch_outbound(msg)
        [submit_sm1, submit_sm2] = yield self.fake_smsc.await_pdus(2)

        # Our multipart_info Redis hash should contain the number of parts and
        # have an appropriate TTL.
        mstash = transport.message_stash
        multipart_info = yield mstash.get_multipart_info(msg['message_id'])
        self.assertEqual(multipart_info, {"parts": "2"})
        mpi_ttl = yield mstash.redis.ttl(multipart_info_key(msg['message_id']))
        self.assertTrue(
            mpi_ttl <= mstash.config.submit_sm_expiry,
            "mpi_ttl (%s) > submit_sm_expiry (%s)" % (
                mpi_ttl, mstash.config.submit_sm_expiry))

        # We get one response per PDU, so we only send the nack after receiving
        # both responses.
        self.fake_smsc.send_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm1),
                         message_id='foo', command_status='ESME_RSUBMITFAIL'))
        self.fake_smsc.send_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm2), message_id='bar'))
        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'nack')
        self.assertEqual(event['user_message_id'], msg['message_id'])

        # After all parts are acknowledged, our multipart_info hash should have
        # the details of the responses and a much shorter TTL.
        mstash = transport.message_stash
        multipart_info = yield mstash.get_multipart_info(msg['message_id'])
        self.assertEqual(multipart_info, {
            "parts": "2",
            "event_counter": "2",
            "part:foo": "fail",
            "part:bar": "ack",
            "event_result": "fail",
        })
        mpi_ttl = yield mstash.redis.ttl(multipart_info_key(msg['message_id']))
        self.assertTrue(
            mpi_ttl <= mstash.config.completed_multipart_info_expiry,
            "mpi_ttl (%s) > completed_multipart_info_expiry (%s)" % (
                mpi_ttl, mstash.config.completed_multipart_info_expiry))

    @inlineCallbacks
    def test_mt_sms_multipart_fail_second_part(self):
        yield self.get_transport({
            'submit_short_message_processor_config': {
                'send_multipart_udh': True,
            }
        })
        content = '1' * 161
        msg = self.tx_helper.make_outbound(content)
        yield self.tx_helper.dispatch_outbound(msg)
        [submit_sm1, submit_sm2] = yield self.fake_smsc.await_pdus(2)
        self.fake_smsc.send_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm1), message_id='foo'))
        self.fake_smsc.send_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm2),
                         message_id='bar', command_status='ESME_RSUBMITFAIL'))
        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'nack')
        self.assertEqual(event['user_message_id'], msg['message_id'])

    @inlineCallbacks
    def test_mt_sms_multipart_fail_no_remote_id(self):
        yield self.get_transport({
            'submit_short_message_processor_config': {
                'send_multipart_udh': True,
            }
        })
        content = '1' * 161
        msg = self.tx_helper.make_outbound(content)
        yield self.tx_helper.dispatch_outbound(msg)
        [submit_sm1, submit_sm2] = yield self.fake_smsc.await_pdus(2)
        self.fake_smsc.send_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm1),
                         message_id='', command_status='ESME_RINVDSTADR'))
        self.fake_smsc.send_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm2),
                         message_id='', command_status='ESME_RINVDSTADR'))
        [event] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(event['event_type'], 'nack')
        self.assertEqual(event['user_message_id'], msg['message_id'])

    @inlineCallbacks
    def test_message_persistence(self):
        transport = yield self.get_transport()
        message_stash = transport.message_stash
        config = transport.get_static_config()

        msg = self.tx_helper.make_outbound("hello world")
        yield message_stash.cache_message(msg)

        ttl = yield transport.redis.ttl(message_key(msg['message_id']))
        self.assertTrue(0 < ttl <= config.submit_sm_expiry)

        retrieved_msg = yield message_stash.get_cached_message(
            msg['message_id'])
        self.assertEqual(msg, retrieved_msg)
        yield message_stash.delete_cached_message(msg['message_id'])
        self.assertEqual(
            (yield message_stash.get_cached_message(msg['message_id'])),
            None)

    @inlineCallbacks
    def test_message_clearing(self):
        transport = yield self.get_transport()
        message_stash = transport.message_stash
        msg = self.tx_helper.make_outbound('hello world')
        yield message_stash.set_sequence_number_message_id(
            3, msg['message_id'])
        yield message_stash.cache_message(msg)
        yield self.fake_smsc.handle_pdu(SubmitSMResp(
            sequence_number=3, message_id='foo', command_status='ESME_ROK'))
        self.assertEqual(
            None,
            (yield message_stash.get_cached_message(msg['message_id'])))

    @inlineCallbacks
    def test_sequence_number_persistence(self):
        """
        We create sequence_number to message_id mappings with an appropriate
        TTL and can delete them when we're done.
        """
        transport = yield self.get_transport()
        message_stash = transport.message_stash
        config = transport.get_static_config()

        yield message_stash.set_sequence_number_message_id(12, "abc")
        ttl = yield transport.redis.ttl(sequence_number_key(12))
        self.assertTrue(0 < ttl <= config.submit_sm_expiry)

        message_id = yield message_stash.get_sequence_number_message_id(12)
        self.assertEqual(message_id, "abc")

        yield message_stash.delete_sequence_number_message_id(12)
        message_id = yield message_stash.get_sequence_number_message_id(12)
        self.assertEqual(message_id, None)

    @inlineCallbacks
    def test_sequence_number_clearing(self):
        """
        When we finish processing a PDU response, the mapping gets deleted.
        """
        transport = yield self.get_transport()
        message_stash = transport.message_stash

        yield message_stash.set_sequence_number_message_id(37, "def")
        message_id = yield message_stash.get_sequence_number_message_id(37)
        self.assertEqual(message_id, "def")

        yield self.fake_smsc.handle_pdu(SubmitSMResp(
            sequence_number=37, message_id='foo', command_status='ESME_ROK'))
        message_id = yield message_stash.get_sequence_number_message_id(37)
        self.assertEqual(message_id, None)

    @inlineCallbacks
    def test_link_remote_message_id(self):
        transport = yield self.get_transport()
        config = transport.get_static_config()

        msg = self.tx_helper.make_outbound('hello world')
        yield self.tx_helper.dispatch_outbound(msg)

        pdu = yield self.fake_smsc.await_pdu()
        yield self.fake_smsc.handle_pdu(
            SubmitSMResp(sequence_number=seq_no(pdu),
                         message_id='foo',
                         command_status='ESME_ROK'))
        self.assertEqual(
            msg['message_id'],
            (yield transport.message_stash.get_internal_message_id('foo')))

        ttl = yield transport.redis.ttl(remote_message_key('foo'))
        self.assertTrue(0 < ttl <= config.third_party_id_expiry)

    @inlineCallbacks
    def test_out_of_order_responses(self):
        yield self.get_transport()
        yield self.tx_helper.make_dispatch_outbound("msg 1", message_id='444')
        submit_sm1 = yield self.fake_smsc.await_pdu()
        response1 = SubmitSMResp(seq_no(submit_sm1), "3rd_party_id_1")

        yield self.tx_helper.make_dispatch_outbound("msg 2", message_id='445')
        submit_sm2 = yield self.fake_smsc.await_pdu()
        response2 = SubmitSMResp(seq_no(submit_sm2), "3rd_party_id_2")

        # respond out of order - just to keep things interesting
        yield self.fake_smsc.handle_pdu(response2)
        yield self.fake_smsc.handle_pdu(response1)

        [ack1, ack2] = yield self.tx_helper.wait_for_dispatched_events(2)
        self.assertEqual(ack1['user_message_id'], '445')
        self.assertEqual(ack1['sent_message_id'], '3rd_party_id_2')
        self.assertEqual(ack2['user_message_id'], '444')
        self.assertEqual(ack2['sent_message_id'], '3rd_party_id_1')

    @inlineCallbacks
    def test_delivery_report_for_unknown_message(self):
        dr = self.DR_TEMPLATE % ('foo',)
        deliver = DeliverSM(1, short_message=dr, esm_class=4)
        yield self.get_transport()
        with LogCatcher(message="Failed to retrieve message id") as lc:
            yield self.fake_smsc.handle_pdu(deliver)
            [warning] = lc.logs
            self.assertEqual(warning['message'],
                             ("Failed to retrieve message id for delivery "
                              "report. Delivery report from %s "
                              "discarded." % self.tx_helper.transport_name,))

    @inlineCallbacks
    def test_delivery_report_delivered_delete_stored_remote_id(self):
        transport = yield self.get_transport({
            'final_dr_third_party_id_expiry': 23,
        })

        yield transport.message_stash.set_remote_message_id('bar', 'foo')
        remote_id_ttl = yield transport.redis.ttl(remote_message_key('foo'))

        self.assertTrue(
            remote_id_ttl > 23,
            "remote_id_ttl (%s) <= final_dr_third_party_id_expiry (23)"
            % (remote_id_ttl,))

        pdu = DeliverSM(sequence_number=1, esm_class=4)
        pdu.add_optional_parameter('receipted_message_id', 'foo')
        pdu.add_optional_parameter('message_state', 2)
        yield self.fake_smsc.handle_pdu(pdu)

        yield self.tx_helper.wait_for_dispatched_events(1)

        remote_id_ttl = yield transport.redis.ttl(remote_message_key('foo'))

        self.assertTrue(
            remote_id_ttl <= 23,
            "remote_id_ttl (%s) > final_dr_third_party_id_expiry (23)"
            % (remote_id_ttl,))

    @inlineCallbacks
    def test_delivery_report_failed_delete_stored_remote_id(self):
        transport = yield self.get_transport({
            'final_dr_third_party_id_expiry': 23,
        })

        yield transport.message_stash.set_remote_message_id('bar', 'foo')

        remote_id_ttl = yield transport.redis.ttl(remote_message_key('foo'))

        self.assertTrue(
            remote_id_ttl > 23,
            "remote_id_ttl (%s) <= final_dr_third_party_id_expiry (23)"
            % (remote_id_ttl,))

        pdu = DeliverSM(sequence_number=1, esm_class=4)
        pdu.add_optional_parameter('receipted_message_id', 'foo')
        pdu.add_optional_parameter('message_state', 8)
        yield self.fake_smsc.handle_pdu(pdu)

        yield self.tx_helper.wait_for_dispatched_events(1)

        remote_id_ttl = yield transport.redis.ttl(remote_message_key('foo'))

        self.assertTrue(
            remote_id_ttl <= 23,
            "remote_id_ttl (%s) > final_dr_third_party_id_expiry (23)"
            % (remote_id_ttl,))

    @inlineCallbacks
    def test_delivery_report_pending_keep_stored_remote_id(self):
        transport = yield self.get_transport({
            'final_dr_third_party_id_expiry': 23,
        })

        yield transport.message_stash.set_remote_message_id('bar', 'foo')

        remote_id_ttl = yield transport.redis.ttl(remote_message_key('foo'))

        self.assertTrue(
            remote_id_ttl > 23,
            "remote_id_ttl (%s) <= final_dr_third_party_id_expiry (23)"
            % (remote_id_ttl,))

        pdu = DeliverSM(sequence_number=1, esm_class=4)
        pdu.add_optional_parameter('receipted_message_id', 'foo')
        pdu.add_optional_parameter('message_state', 1)
        yield self.fake_smsc.handle_pdu(pdu)

        yield self.tx_helper.wait_for_dispatched_events(1)

        remote_id_ttl = yield transport.redis.ttl(remote_message_key('foo'))

        self.assertTrue(
            remote_id_ttl > 23,
            "remote_id_ttl (%s) <= final_dr_third_party_id_expiry (23)"
            % (remote_id_ttl,))

    @inlineCallbacks
    def test_reconnect(self):
        transport = yield self.get_transport(bind=False)
        connector = transport.connectors[transport.transport_name]
        # Unbound and disconnected.
        self.assertEqual(connector._consumers['outbound'].paused, True)
        # Connect and bind.
        yield self.fake_smsc.bind()
        self.assertEqual(connector._consumers['outbound'].paused, False)
        # Disconnect.
        yield self.fake_smsc.disconnect()
        self.assertEqual(connector._consumers['outbound'].paused, True)
        # Wait for reconnect, but don't bind.
        self.clock.advance(transport.service.delay)
        yield self.fake_smsc.await_connected()
        self.assertEqual(connector._consumers['outbound'].paused, True)
        # Bind.
        yield self.fake_smsc.bind()
        self.assertEqual(connector._consumers['outbound'].paused, False)

    @inlineCallbacks
    def test_bind_params(self):
        yield self.get_transport({
            'system_id': 'myusername',
            'password': 'mypasswd',
            'system_type': 'SMPP',
            'interface_version': '33',
            'address_range': '*12345',
        }, bind=False)
        bind_pdu = yield self.fake_smsc.await_pdu()
        # This test runs for multiple bind types, so we only assert on the
        # common prefix of the command.
        self.assertEqual(bind_pdu['header']['command_id'][:5], 'bind_')
        self.assertEqual(bind_pdu['body'], {'mandatory_parameters': {
            'system_id': 'myusername',
            'password': 'mypasswd',
            'system_type': 'SMPP',
            'interface_version': '33',
            'address_range': '*12345',
            'addr_ton': 'unknown',
            'addr_npi': 'unknown',
        }})

    @inlineCallbacks
    def test_bind_params_long_password(self):
        lc = LogCatcher(message="Password longer than 8 characters,")
        with lc:
            yield self.get_transport({
                'worker_name': 'sphex',
                'system_id': 'myusername',
                'password': 'mypass789',
                'system_type': 'SMPP',
                'interface_version': '33',
                'address_range': '*12345',
            }, bind=False)
            bind_pdu = yield self.fake_smsc.await_pdu()
        # This test runs for multiple bind types, so we only assert on the
        # common prefix of the command.
        self.assertEqual(bind_pdu['header']['command_id'][:5], 'bind_')
        self.assertEqual(bind_pdu['body'], {'mandatory_parameters': {
            'system_id': 'myusername',
            'password': 'mypass78',
            'system_type': 'SMPP',
            'interface_version': '33',
            'address_range': '*12345',
            'addr_ton': 'unknown',
            'addr_npi': 'unknown',
        }})

        # Check that the truncation was logged.
        [warning] = lc.logs
        expected_msg = "Password longer than 8 characters, truncating."
        self.assertEqual(warning['message'], (expected_msg,))
        for l in lc.logs:
            self.assertEqual(l['system'], 'sphex')

    @inlineCallbacks
    def test_default_bind_params(self):
        yield self.get_transport(bind=False)
        bind_pdu = yield self.fake_smsc.await_pdu()
        # This test runs for multiple bind types, so we only assert on the
        # common prefix of the command.
        self.assertEqual(bind_pdu['header']['command_id'][:5], 'bind_')
        self.assertEqual(bind_pdu['body'], {'mandatory_parameters': {
            'system_id': 'foo',  # Mandatory param, defaulted by helper.
            'password': 'bar',   # Mandatory param, defaulted by helper.
            'system_type': '',
            'interface_version': '34',
            'address_range': '',
            'addr_ton': 'unknown',
            'addr_npi': 'unknown',
        }})

    @inlineCallbacks
    def test_startup_with_backlog(self):
        yield self.get_transport(bind=False)

        # Disconnected.
        for i in range(2):
            msg = self.tx_helper.make_outbound('hello world %s' % (i,))
            yield self.tx_helper.dispatch_outbound(msg)

        # Connect and bind.
        yield self.fake_smsc.bind()
        [submit_sm1, submit_sm2] = yield self.fake_smsc.await_pdus(2)
        self.assertEqual(short_message(submit_sm1), 'hello world 0')
        self.assertEqual(short_message(submit_sm2), 'hello world 1')

    @inlineCallbacks
    def test_starting_status(self):
        """
        The SMPP bind process emits three status events.
        """
        yield self.get_transport({'publish_status': True})
        msgs = yield self.tx_helper.wait_for_dispatched_statuses()
        [msg_starting, msg_binding, msg_bound] = msgs

        self.assertEqual(msg_starting['status'], 'down')
        self.assertEqual(msg_starting['component'], 'smpp')
        self.assertEqual(msg_starting['type'], 'starting')
        self.assertEqual(msg_starting['message'], 'Starting')

        self.assertEqual(msg_binding['status'], 'down')
        self.assertEqual(msg_binding['component'], 'smpp')
        self.assertEqual(msg_binding['type'], 'binding')
        self.assertEqual(msg_binding['message'], 'Binding')

        self.assertEqual(msg_bound['status'], 'ok')
        self.assertEqual(msg_bound['component'], 'smpp')
        self.assertEqual(msg_bound['type'], 'bound')
        self.assertEqual(msg_bound['message'], 'Bound')

    @inlineCallbacks
    def test_connect_status(self):
        transport = yield self.get_transport(
            {'publish_status': True}, bind=False)

        # disconnect
        yield self.fake_smsc.disconnect()
        self.tx_helper.clear_dispatched_statuses()

        # reconnect
        self.clock.advance(transport.service.delay)
        yield self.fake_smsc.await_connected()

        [msg] = yield self.tx_helper.wait_for_dispatched_statuses()
        self.assertEqual(msg['status'], 'down')
        self.assertEqual(msg['component'], 'smpp')
        self.assertEqual(msg['type'], 'binding')
        self.assertEqual(msg['message'], 'Binding')

    @inlineCallbacks
    def test_unbinding_status(self):
        transport = yield self.get_transport({'publish_status': True})
        self.tx_helper.clear_dispatched_statuses()
        yield transport.service.get_protocol().unbind()

        [msg] = yield self.tx_helper.wait_for_dispatched_statuses()
        self.assertEqual(msg['status'], 'down')
        self.assertEqual(msg['component'], 'smpp')
        self.assertEqual(msg['type'], 'unbinding')
        self.assertEqual(msg['message'], 'Unbinding')

    @inlineCallbacks
    def test_bind_status(self):
        yield self.get_transport({'publish_status': True}, bind=False)

        self.tx_helper.clear_dispatched_statuses()

        yield self.fake_smsc.bind()

        [msg] = yield self.tx_helper.wait_for_dispatched_statuses()
        self.assertEqual(msg['status'], 'ok')
        self.assertEqual(msg['component'], 'smpp')
        self.assertEqual(msg['type'], 'bound')
        self.assertEqual(msg['message'], 'Bound')

    @inlineCallbacks
    def test_bind_timeout_status(self):
        yield self.get_transport({
            'publish_status': True,
            'smpp_bind_timeout': 3,
        }, bind=False)

        # wait for bind pdu
        yield self.fake_smsc.await_pdu()

        self.tx_helper.clear_dispatched_statuses()
        self.clock.advance(3)

        [msg] = yield self.tx_helper.wait_for_dispatched_statuses()
        self.assertEqual(msg['status'], 'down')
        self.assertEqual(msg['component'], 'smpp')
        self.assertEqual(msg['type'], 'bind_timeout')
        self.assertEqual(msg['message'], 'Timed out awaiting bind')

        yield self.fake_smsc.disconnect()

    @inlineCallbacks
    def test_connection_lost_status(self):
        yield self.get_transport({'publish_status': True})

        self.tx_helper.clear_dispatched_statuses()
        yield self.fake_smsc.disconnect()

        [msg] = yield self.tx_helper.wait_for_dispatched_statuses()
        self.assertEqual(msg['status'], 'down')
        self.assertEqual(msg['status'], 'down')
        self.assertEqual(msg['component'], 'smpp')
        self.assertEqual(msg['type'], 'connection_lost')

        self.assertEqual(
            msg['message'],
            'Connection was closed cleanly: Connection done.')

    @inlineCallbacks
    def test_smsc_throttle_status(self):
        yield self.get_transport({
            'publish_status': True,
            'throttle_delay': 3
        })

        self.tx_helper.clear_dispatched_statuses()

        msg = self.tx_helper.make_outbound("throttle me")
        yield self.tx_helper.dispatch_outbound(msg)
        submit_sm_pdu = yield self.fake_smsc.await_pdu()

        yield self.fake_smsc.handle_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm_pdu),
                         message_id='foo',
                         command_status='ESME_RTHROTTLED'))

        [msg] = yield self.tx_helper.wait_for_dispatched_statuses()
        self.assertEqual(msg['status'], 'degraded')
        self.assertEqual(msg['component'], 'smpp')
        self.assertEqual(msg['type'], 'throttled')
        self.assertEqual(msg['message'], 'Throttled')

        self.tx_helper.clear_dispatched_statuses()
        self.clock.advance(3)

        submit_sm_pdu_retry = yield self.fake_smsc.await_pdu()
        yield self.fake_smsc.handle_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm_pdu_retry),
                         message_id='bar',
                         command_status='ESME_ROK'))

        self.clock.advance(0)

        [msg] = yield self.tx_helper.wait_for_dispatched_statuses()
        self.assertEqual(msg['status'], 'ok')
        self.assertEqual(msg['component'], 'smpp')
        self.assertEqual(msg['type'], 'throttled_end')
        self.assertEqual(msg['message'], 'No longer throttled')

    @inlineCallbacks
    def test_smsc_throttle_reconnect_status(self):
        transport = yield self.get_transport({
            'publish_status': True,
        })

        self.tx_helper.clear_dispatched_statuses()

        msg = self.tx_helper.make_outbound("throttle me")
        yield self.tx_helper.dispatch_outbound(msg)
        submit_sm_pdu = yield self.fake_smsc.await_pdu()

        yield self.fake_smsc.handle_pdu(
            SubmitSMResp(sequence_number=seq_no(submit_sm_pdu),
                         message_id='foo',
                         command_status='ESME_RTHROTTLED'))

        yield self.fake_smsc.disconnect()
        self.tx_helper.clear_dispatched_statuses()
        self.clock.advance(transport.service.delay)

        yield self.fake_smsc.bind()

        msgs = yield self.tx_helper.wait_for_dispatched_statuses()
        [msg1, msg2, msg3] = msgs

        self.assertEqual(msg1['type'], 'binding')
        self.assertEqual(msg2['type'], 'bound')

        self.assertEqual(msg3['status'], 'degraded')
        self.assertEqual(msg3['component'], 'smpp')
        self.assertEqual(msg3['type'], 'throttled')
        self.assertEqual(msg3['message'], 'Throttled')

    @inlineCallbacks
    def test_tps_throttle_status(self):
        yield self.get_transport({
            'publish_status': True,
            'mt_tps': 2
        })

        self.tx_helper.clear_dispatched_statuses()

        yield self.tx_helper.make_dispatch_outbound('hello world 1')
        yield self.tx_helper.make_dispatch_outbound('hello world 2')
        self.tx_helper.make_dispatch_outbound('hello world 3')
        yield self.fake_smsc.await_pdus(2)

        # We can't wait here because that requires throttling to end.
        [msg] = self.tx_helper.get_dispatched_statuses()
        self.assertEqual(msg['status'], 'degraded')
        self.assertEqual(msg['component'], 'smpp')
        self.assertEqual(msg['type'], 'throttled')
        self.assertEqual(msg['message'], 'Throttled')

        self.tx_helper.clear_dispatched_statuses()

        self.clock.advance(1)

        [msg] = yield self.tx_helper.wait_for_dispatched_statuses()
        self.assertEqual(msg['status'], 'ok')
        self.assertEqual(msg['component'], 'smpp')
        self.assertEqual(msg['type'], 'throttled_end')
        self.assertEqual(msg['message'], 'No longer throttled')

    @inlineCallbacks
    def test_tps_throttle_reconnect_status(self):
        transport = yield self.get_transport({
            'publish_status': True,
            'mt_tps': 2
        })

        self.tx_helper.clear_dispatched_statuses()

        yield self.tx_helper.make_dispatch_outbound('hello world 1')
        yield self.tx_helper.make_dispatch_outbound('hello world 2')
        self.tx_helper.make_dispatch_outbound('hello world 3')
        yield self.fake_smsc.await_pdus(2)

        yield self.fake_smsc.disconnect()
        self.tx_helper.clear_dispatched_statuses()
        self.clock.advance(transport.service.delay)

        yield self.fake_smsc.bind()

        msgs = yield self.tx_helper.wait_for_dispatched_statuses()
        [msg1, msg2, msg3] = msgs

        self.assertEqual(msg1['type'], 'binding')
        self.assertEqual(msg2['type'], 'bound')

        self.assertEqual(msg3['status'], 'degraded')
        self.assertEqual(msg3['component'], 'smpp')
        self.assertEqual(msg3['type'], 'throttled')
        self.assertEqual(msg3['message'], 'Throttled')


class SmppTransmitterTransportTestCase(SmppTransceiverTransportTestCase):
    transport_class = SmppTransmitterTransport


class SmppReceiverTransportTestCase(SmppTransceiverTransportTestCase):
    transport_class = SmppReceiverTransport


class SmppTransceiverTransportWithOldConfigTestCase(
        SmppTransceiverTransportTestCase):

    transport_class = SmppTransceiverTransportWithOldConfig

    def setUp(self):
        self.clock = Clock()
        self.fake_smsc = FakeSMSC()
        self.tx_helper = self.add_helper(TransportHelper(self.transport_class))
        self.default_config = {
            'transport_name': self.tx_helper.transport_name,
            'worker_name': self.tx_helper.transport_name,
            'twisted_endpoint': self.fake_smsc.endpoint,
            'system_id': 'foo',
            'password': 'bar',
            'data_coding_overrides': {
                0: 'utf-8',
            }
        }

    def _get_transport_config(self, config):
        """
        The test cases assume the new config, this flattens the
        config key word arguments value to match an old config
        layout without the processor configs.
        """

        cfg = self.default_config.copy()

        processor_config_keys = [
            'submit_short_message_processor_config',
            'deliver_short_message_processor_config',
            'delivery_report_processor_config',
        ]

        for config_key in processor_config_keys:
            processor_config = config.pop(config_key, {})
            for name, value in processor_config.items():
                cfg[name] = value

        # Update with all remaining (non-processor) config values
        cfg.update(config)
        return cfg


class TataUssdSmppTransportTestCase(SmppTransportTestCase):

    transport_class = SmppTransceiverTransport

    @inlineCallbacks
    def test_submit_and_deliver_ussd_continue(self):
        yield self.get_transport()

        yield self.tx_helper.make_dispatch_outbound(
            "hello world", transport_type="ussd")

        submit_sm_pdu = yield self.fake_smsc.await_pdu()
        self.assertEqual(command_id(submit_sm_pdu), 'submit_sm')
        self.assertEqual(pdu_tlv(submit_sm_pdu, 'ussd_service_op'), '02')
        self.assertEqual(pdu_tlv(submit_sm_pdu, 'its_session_info'), '0000')

        # Server delivers a USSD message to the Client
        pdu = DeliverSM(seq_no(submit_sm_pdu) + 1, short_message="reply!")
        pdu.add_optional_parameter('ussd_service_op', '02')
        pdu.add_optional_parameter('its_session_info', '0000')

        yield self.fake_smsc.handle_pdu(pdu)

        [mess] = yield self.tx_helper.wait_for_dispatched_inbound(1)

        self.assertEqual(mess['content'], "reply!")
        self.assertEqual(mess['transport_type'], "ussd")
        self.assertEqual(mess['session_event'],
                         TransportUserMessage.SESSION_RESUME)

    @inlineCallbacks
    def test_submit_and_deliver_ussd_close(self):
        yield self.get_transport()

        yield self.tx_helper.make_dispatch_outbound(
            "hello world", transport_type="ussd",
            session_event=TransportUserMessage.SESSION_CLOSE)

        submit_sm_pdu = yield self.fake_smsc.await_pdu()
        self.assertEqual(command_id(submit_sm_pdu), 'submit_sm')
        self.assertEqual(pdu_tlv(submit_sm_pdu, 'ussd_service_op'), '02')
        self.assertEqual(pdu_tlv(submit_sm_pdu, 'its_session_info'), '0001')

        # Server delivers a USSD message to the Client
        pdu = DeliverSM(seq_no(submit_sm_pdu) + 1, short_message="reply!")
        pdu.add_optional_parameter('ussd_service_op', '02')
        pdu.add_optional_parameter('its_session_info', '0001')

        yield self.fake_smsc.handle_pdu(pdu)

        [mess] = yield self.tx_helper.wait_for_dispatched_inbound(1)

        self.assertEqual(mess['content'], "reply!")
        self.assertEqual(mess['transport_type'], "ussd")
        self.assertEqual(mess['session_event'],
                         TransportUserMessage.SESSION_CLOSE)


class TestSubmitShortMessageProcessorConfig(VumiTestCase):

    def get_config(self, config_dict):
        return SubmitShortMessageProcessor.CONFIG_CLASS(config_dict)

    def assert_config_error(self, config_dict):
        try:
            self.get_config(config_dict)
            self.fail("ConfigError not raised.")
        except ConfigError as err:
            return err.args[0]

    def test_long_message_params(self):
        self.get_config({})
        self.get_config({'send_long_messages': True})
        self.get_config({'send_multipart_sar': True})
        self.get_config({'send_multipart_udh': True})
        errmsg = self.assert_config_error({
            'send_long_messages': True,
            'send_multipart_sar': True,
        })
        self.assertEqual(errmsg, (
            "The following parameters are mutually exclusive: "
            "send_long_messages, send_multipart_sar"))
        errmsg = self.assert_config_error({
            'send_long_messages': True,
            'send_multipart_sar': True,
            'send_multipart_udh': True,
        })
        self.assertEqual(errmsg, (
            "The following parameters are mutually exclusive: "
            "send_long_messages, send_multipart_sar, send_multipart_udh"))



# -*- test-case-name: vumi.transports.devnull.tests.test_devnull -*-
import random
import uuid

from vumi import log

from vumi.transports.base import Transport
from vumi.message import TransportUserMessage

from twisted.internet.defer import inlineCallbacks


class DevNullTransport(Transport):
    """
    DevNullTransport for messages that need fake delivery to networks.
    Useful for testing.

    Configuration parameters:

    :type transport_type: str
    :param transport_type:
        The transport type to emulate, defaults to sms.
    :type ack_rate: float
    :param ack_rate:
        How many messages should be ack'd. The remainder will be nacked.
        The `failure_rate` and `reply_rate` treat the `ack_rate` as 100%.
    :type failure_rate: float
    :param failure_rate:
        How many messages should be treated as failures.
        Float value between 0.0 and 1.0.
    :type reply_rate: float
    :param reply_rate:
        For how many messages should we generate a reply?
        Float value between 0.0 and 1.0.
    :type reply_copy: str
    :param reply_copy:
        What copy should be sent as the reply, defaults to echo-ing the content
        of the outbound message.
    """

    def validate_config(self):
        self.transport_type = self.config.get('transport_type', 'sms')
        self.ack_rate = float(self.config['ack_rate'])
        self.failure_rate = float(self.config['failure_rate'])
        self.reply_rate = float(self.config['reply_rate'])
        self.reply_copy = self.config.get('reply_copy')

    def setup_transport(self):
        pass

    def teardown_transport(self):
        pass

    @inlineCallbacks
    def handle_outbound_message(self, message):
        if random.random() > self.ack_rate:
            yield self.publish_nack(message['message_id'],
                'Not accepted by network')
            return

        dr = ('failed' if random.random() < self.failure_rate
                else 'delivered')
        log.info('MT %(dr)s: %(from_addr)s -> %(to_addr)s: %(content)s' % {
            'dr': dr,
            'from_addr': message['from_addr'],
            'to_addr': message['to_addr'],
            'content': message['content'],
            })
        yield self.publish_ack(message['message_id'],
            sent_message_id=uuid.uuid4().hex)
        yield self.publish_delivery_report(message['message_id'], dr)
        if random.random() < self.reply_rate:
            reply_copy = self.reply_copy or message['content']
            log.info('MO %(from_addr)s -> %(to_addr)s: %(content)s' % {
                'from_addr': message['to_addr'],
                'to_addr': message['from_addr'],
                'content': reply_copy,
                })
            yield self.publish_message(
                message_id=uuid.uuid4().hex,
                content=reply_copy,
                to_addr=message['from_addr'],
                from_addr=message['to_addr'],
                provider='devnull',
                session_event=TransportUserMessage.SESSION_NONE,
                transport_type=self.transport_type,
                transport_metadata={})
